
from pyvisdk.base.managed_object_types import ManagedObjectTypes

from pyvisdk.base.base_entity import BaseEntity

import logging

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

class ProfileComplianceManager(BaseEntity):
    '''Interface handling the Compliance aspects of entities.'''

    def __init__(self, core, name=None, ref=None, type=ManagedObjectTypes.ProfileComplianceManager):
        super(ProfileComplianceManager, self).__init__(core, name=name, ref=ref, type=type)

    

    
    
    def CheckCompliance_Task(self, profile=None, entity=None):
        '''Check compliance of an entity against a Profile.
        
        :param profile: If specified, check compliance against the specified profiles. If not specified, use the profiles associated with the entities. If both Profiles and Entities are specified, Check the compliance of each Entity against each of the profile specified. For more information, look at the KMap below. P represents if Profile is specified. E represents if Entity is specified.
        
        :param entity: If specified, the compliance check is done against this entity.
        
        '''
        return self.delegate("CheckCompliance_Task")(profile, entity)
    
    def ClearComplianceStatus(self, profile=None, entity=None):
        '''Clear the saved ComplianceResult based on profile and entity filtering
        criteria.
        
        :param profile: If specified, clear the ComplianceResult related to the Profile.
        
        :param entity: If specified, clear the ComplianceResult related to the entity. If profile and entity are not specified, all the ComplianceResults will be cleared.
        
        '''
        return self.delegate("ClearComplianceStatus")(profile, entity)
    
    def QueryComplianceStatus(self, profile=None, entity=None):
        '''Query the compliance status based on Profile and Entity filter.
        
        :param profile: If specified, compliance result for the specified profiles will be returned. This acts like a filtering criteria for the ComplianceResults based on specified profiles.
        
        :param entity: If specified, compliance results for these entities will be returned. This acts like a filtering criteria for the ComplianceResults based on entities.
        
        '''
        return self.delegate("QueryComplianceStatus")(profile, entity)
    
    def QueryExpressionMetadata(self, expressionName=None, profile=None):
        '''Query the metadata for the expressions.
        
        :param expressionName: Names of the Expressions for which metadata is requested. If expressionNames are not specified, metadata for all known expressions is returned
        
        :param profile: Base profile whose context needs to be used during the operationvSphere API 5.0
        
        '''
        return self.delegate("QueryExpressionMetadata")(expressionName, profile)



from pyvisdk.esxcli.executer import execute_soap
from pyvisdk.esxcli.base import Base

class StorageCorePluginRegistration(Base):
    '''
    Operations pertaining to the pluggable storage architectures' plugin registration. The operations currently allowed are to list registered plugins on the system with the type of those plugins and to register and unregister plugin modules on the system
    '''
    moid = 'ha-cli-handler-storage-core-plugin-registration'
    def add(self, modulename, pluginclass, pluginname, dependencies=None, fullpath=None):
        '''
        Register a plugin module with PSA.
        :param dependencies: string, Add the [optional] dependencies for this module to loaded
        :param fullpath: string, Add the [optional] full path to this module
        :param modulename: string, Select the module name to be registered
        :param pluginclass: string, Indicate the class of plugin to register.  Allowed values are MP, VAAI or MPP defined subplugins like PSP, SATP.
        :param pluginname: string, Select the plugin name to be registered
        :returns: void
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.storage.core.plugin.registration.Add',
                            dependencies=dependencies,
                            fullpath=fullpath,
                            modulename=modulename,
                            pluginclass=pluginclass,
                            pluginname=pluginname,
                            )
    def list(self, modulename=None, pluginclass=None):
        '''
        List modules currently registered with PSA.
        :param modulename: string, Filter the output of this command to only show a single module.
        :param pluginclass: string, Indicate the class of plugin to list.  Allowed values are MP, VAAI or MPP defined subplugins like PSP, SATP.
        :returns: vim.EsxCLI.storage.core.plugin.registration.list.RegisteredModuleList[]
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.storage.core.plugin.registration.List',
                            modulename=modulename,
                            pluginclass=pluginclass,
                            )
    def remove(self, modulename):
        '''
        UnRegister a plugin module with PSA.
        :param modulename: string, Select the module name to be unregistered
        :returns: void
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.storage.core.plugin.registration.Remove',
                            modulename=modulename,
                            )   



import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def ClusterDasVmSettings(vim, *args, **kwargs):
    '''The ClusterDasVmSettings data object contains the HA configuration settings
    specified for a single virtual machine (identified by
    ClusterDasVmConfigInfo.key) or as cluster-wide defaults
    (ClusterDasConfigInfo.defaultVmSettings).All fields are optional. If you set
    the parameter to when you call ReconfigureComputeResource_Task, an unset
    property has no effect on the existing property value in the cluster
    configuration on the Server. If you set the parameter to when you reconfigure a
    cluster, the cluster configuration is reverted to the default values, then the
    new configuration values are applied.'''

    obj = vim.client.factory.create('{urn:vim25}ClusterDasVmSettings')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 0:
        raise IndexError('Expected at least 1 arguments got: %d' % len(args))

    required = [  ]
    optional = [ 'isolationResponse', 'restartPriority', 'vmToolsMonitoringSettings',
        'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

HostFirewallRuleProtocol = Enum(
    'tcp',
    'udp',

)




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

FilterSpecComparisonOperator = Enum(
    'beginsWith',
    'contains',
    'endsWith',
    'equal',
    'greaterThanOrEqual',
    'lessThanOrEqual',
    'notContains',
    'notEqual',

)




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

EventAlarmExpressionComparisonOperator = Enum(
    'doesNotEndWith',
    'doesNotStartWith',
    'endsWith',
    'equals',
    'notEqualTo',
    'startsWith',

)




import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def CustomizationGuiUnattended(vim, *args, **kwargs):
    '''The GuiUnattended type maps to the GuiUnattended key in the answer file. These
    values are plugged directly into the file that VirtualCenter stores on the
    target virtual disk. For more detailed information, see the document .'''

    obj = vim.client.factory.create('{urn:vim25}CustomizationGuiUnattended')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 3:
        raise IndexError('Expected at least 4 arguments got: %d' % len(args))

    required = [ 'autoLogon', 'autoLogonCount', 'timeZone' ]
    optional = [ 'password', 'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




from pyvisdk.esxcli.executer import execute_soap
from pyvisdk.esxcli.base import Base

class IscsiAdapterDiscoverySendtarget(Base):
    '''
    Operations that can be performed on iSCSI sendtargets discovery
    '''
    moid = 'ha-cli-handler-iscsi-adapter-discovery-sendtarget'
    def add(self, adapter, address):
        '''
        Add a sendtarget address
        :param adapter: string, The iSCSI adapter name.
        :param address: string, The iSCSI sendtarget address: <ip/dns[:port]>
        :returns: void
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.iscsi.adapter.discovery.sendtarget.Add',
                            adapter=adapter,
                            address=address,
                            )
    def list(self, adapter=None):
        '''
        List sendtarget addresses
        :param adapter: string, The iSCSI adapter name.
        :returns: vim.EsxCLI.iscsi.adapter.discovery.sendtarget.list.Sendtarget[]
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.iscsi.adapter.discovery.sendtarget.List',
                            adapter=adapter,
                            )
    def remove(self, adapter, address):
        '''
        Remove a sendtarget address
        :param adapter: string, The iSCSI adapter name.
        :param address: string, The iSCSI sendtarget address: <ip/dns[:port]>
        :returns: void
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.iscsi.adapter.discovery.sendtarget.Remove',
                            adapter=adapter,
                            address=address,
                            )   



########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

DatastoreAccessible = Enum(
    'False',
    'True',

)




import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def CustomizationSysprepFailed(vim, *args, **kwargs):
    '''Sysprep failed to run in the guest during customization. This will most like
    have been caused by the fact that the wrong sysprep was used for the guest, so
    we include the version information in the event.'''

    obj = vim.client.factory.create('{urn:vim25}CustomizationSysprepFailed')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 7:
        raise IndexError('Expected at least 8 arguments got: %d' % len(args))

    required = [ 'sysprepVersion', 'systemVersion', 'template', 'chainId', 'createdTime', 'key',
        'userName' ]
    optional = [ 'logLocation', 'changeTag', 'computeResource', 'datacenter', 'ds', 'dvs',
        'fullFormattedMessage', 'host', 'net', 'vm', 'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




from pyvisdk.esxcli.executer import execute_soap
from pyvisdk.esxcli.base import Base

class SystemSyslogConfigLogger(Base):
    '''
    Operations for sub-loggers
    '''
    moid = 'ha-cli-handler-system-syslog-config-logger'
    def set(self, id, reset=None, rotate=None, size=None):
        '''
        Set configuration options for a specific sub-logger
        :param id: string, The id of the logger to configure
        :param reset: string, Reset values to default
        :param rotate: long, Number of rotated logs to keep for a specific logger (requires --id)
        :param size: long, Set size of logs before rotation for a specific logger, in KiB (requires --id)
        :returns: boolean
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.system.syslog.config.logger.Set',
                            id=id,
                            reset=reset,
                            rotate=rotate,
                            size=size,
                            )
    def list(self):
        '''
        Show the currently configured sub-loggers
        :returns: vim.EsxCLI.system.syslog.config.logger.list.LoggerConfiguration[]
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.system.syslog.config.logger.List',
                            )   



import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def NetDnsConfigSpec(vim, *args, **kwargs):
    '''Domain Name Server (DNS) Configuration Specification - a data object for
    configuring the RFC 1034 client side DNS settings. TBD: remove this section,
    only for discussing what goes into this object. Place properties here that are
    specific to the RFC/common to all systems. Properties that are platform
    specific should go into a separate config spec. http://technet.microsoft.com
    /en-us/library/cc778792.aspx http://en.wikipedia.org/wiki/Microsoft_DNS'''

    obj = vim.client.factory.create('{urn:vim25}NetDnsConfigSpec')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 0:
        raise IndexError('Expected at least 1 arguments got: %d' % len(args))

    required = [  ]
    optional = [ 'dhcp', 'domainName', 'hostName', 'ipAddress', 'searchDomain',
        'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

VirtualMachinePowerOpType = Enum(
    'hard',
    'preset',
    'soft',

)




from pyvisdk.esxcli.executer import execute_soap
from pyvisdk.esxcli.base import Base

class NetworkIpDnsSearch(Base):
    '''
    Operations pertaining to DNS search domain configuration.
    '''
    moid = 'ha-cli-handler-network-ip-dns-search'
    def add(self, domain):
        '''
        Add a search domain to the list of domains to be searched when trying to resolve an host name on the ESXi host.
        :param domain: string, The string name of a domain to add to the list of search domains.
        :returns: boolean
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.network.ip.dns.search.Add',
                            domain=domain,
                            )
    def list(self):
        '''
        List the search domains currently configured on the ESXi host in the order in which they will be used when searching.
        :returns: vim.EsxCLI.network.ip.dns.search.list.DNSSearchList
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.network.ip.dns.search.List',
                            )
    def remove(self, domain):
        '''
        Remove a search domain from the list of domains to be searched when trying to resolve an host name on the ESXi host.
        :param domain: string, The string name of a domain to remove from the list of search domains.
        :returns: boolean
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.network.ip.dns.search.Remove',
                            domain=domain,
                            )   



import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def HostPlugStoreTopology(vim, *args, **kwargs):
    '''This data object represents the plug-store topology on a host system. Through
    this data object, the storage structure of a system that utilizes the plug-
    store architecture can be presented.The object entity-relationship diagram is
    modeled below:Description and reasoning behind the relationships:When a storage
    device driver is loaded, it claims a PCI device as a host bus adapter. This
    host bus adapter is represented as an Adapter. The PCI device identifier is a
    property on the HostBusAdapter in the Adapter.Once the host bus adapter is on
    the system, the hardware bus is scanned. If a storage Device is found on the
    bus, the communication path to the Device from the the host bus adapter is
    represented by a Path. A Device may have more than one Path. How those Paths
    are composed to create a Device is determined by a storage Plugin.When a
    storage Plugin is loaded, it claims a set of Paths. It groups these Paths into
    a set of Devices. Devices are hence associated with a set of Paths that might
    be used to provide a single logical device such as in the case of multipathing.
    Devices may be also composed of zero Paths meaning that they do not directly
    use a host bus adapter for communication with underlying storage.The purpose of
    this data object is to represent the topology of storage as seen by the base
    plug-store system. There is some overlap with information in other objects such
    as ScsiTopology which is only applicable when a particular "native
    multipathing" plugin is used. This data object provides the complete inventory
    of Devices and Paths. Hence it provides a superset of Device mappings over data
    object such as ScsiTopology and MultipaThe use cases that this data object
    accommodates includes the following non-exhaustive list:'''

    obj = vim.client.factory.create('{urn:vim25}HostPlugStoreTopology')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 0:
        raise IndexError('Expected at least 1 arguments got: %d' % len(args))

    required = [  ]
    optional = [ 'adapter', 'device', 'path', 'plugin', 'target', 'dynamicProperty',
        'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def HostScsiDisk(vim, *args, **kwargs):
    '''This data object type describes a SCSI disk. A SCSI disk contains a partition
    table which can be changed. To change a SCSI disk, use the device name and the
    partition specification.See RetrieveDiskPartitionInfoSee UpdateDiskPartitions'''

    obj = vim.client.factory.create('{urn:vim25}HostScsiDisk')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 7:
        raise IndexError('Expected at least 8 arguments got: %d' % len(args))

    required = [ 'capacity', 'devicePath', 'lunType', 'operationalState', 'uuid', 'deviceName',
        'deviceType' ]
    optional = [ 'ssd', 'alternateName', 'canonicalName', 'capabilities', 'descriptor',
        'displayName', 'durableName', 'key', 'model', 'queueDepth', 'revision',
        'scsiLevel', 'serialNumber', 'standardInquiry', 'vendor', 'vStorageSupport',
        'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj



'''
Created on Mar 8, 2011

@author: eplaster
'''
import suds.sax, logging, types

from pyvisdk.base.managed_object_types import ManagedObjectTypes

log = logging.getLogger(__name__)
log.setLevel(logging.INFO)

class ManagedObjectReference(suds.sudsobject.Property):
    """Custom class to replace the suds generated class, which lacks _type."""
    def __init__(self, _type, value):
        suds.sudsobject.Property.__init__(self, value)
        self._type = _type

class TaskDelegate(object):
    """ delegates to the real method, and waits for the task to complete"""

    def __init__(self, cls, name):
        """ Initializes the delagate with the method to proxy """
        self.__target = getattr(cls.service, name)
        self.name = name
        self.cls = cls

    def __call__(self, *args, **kwargs):
        """ calls original method """
        log.debug("Calling %s" % self.__target.method.name)
        if len(args):
            _list = []
            for x in list(args):
                if x.__class__.__name__ in ManagedObjectTypes:
                    _list.append(x.ref)
                else:
                    _list.append(x)
            _list.insert(0, self.cls.ref)
            args = _list
        else:
            args = (self.cls.ref,)
        rv = self.__target(*args, **kwargs)
        if self.name[-5:] == '_Task' and self.cls.core.wait_for_task:
            log.debug("Waiting for task to Complete")
            self.cls.core.waitForTask(rv)
        if self.__target.method.name in ["RetrieveProperties", "RetrievePropertiesEx"]:
            return rv
        else:
            return self.cls.core._parse_object_content(rv)

    def _return_value(self, rv):
        if isinstance(rv, list):
            return [self._return_value(x) for x in rv]
        if 'suds.sudsobject' in str(rv.__class__):
            new_obj = eval("pyvisdk.do.%s()" % rv.__class__.__name__)
            for attr in dir(rv):
                if attr[0] != '_':
                    if isinstance(attr, suds.sax.text.Text):
                        new_obj.data[str(attr)] = str(eval('rv.%s' % attr))
                    elif 'suds.sudsobject' in str(eval('rv.%s' % attr)):
                        new_obj.data[str(attr)] = self._return_value(eval('rv.%s' % attr))
                    else:
                        new_obj.data[str(attr)] = eval("rv.%s" % attr)
            return new_obj
        else:
            return rv


class BaseEntity(object):
    def __init__(self, core, name=None, ref=None, type=None):
        self.core = core
        self._name = name
        self.ref = ref
        self._type = type

        self.client = core.client
        self.service = core.client.service

        if self.ref:
            self.ref = ManagedObjectReference(self._type, self.ref.value)

            # we have a ref, AND we don't already have a name, we can get the name from it...
            if not self._name:
                self._name = self.ref.value
        else:
            if name:
                self.ref = self.core.getDecendentsByName(self._name)

    @property
    def name(self):
        '''List of custom field definitions that are valid for the object's type. The fields
        are sorted by name.
        '''
        return self.update('name')

    @name.setter
    def name(self, name):
        self._name = name

    def update(self, prop):
        if type(prop) == types.ListType:
            for x in prop:
                self.update(x)

        data = self.core.getObjectProperties(self.ref, prop, parent=self)
        if isinstance(data, list) and len(data) == 1:
            data = data[0]
        return data

    def getDecendants(self, type=getattr(ManagedObjectTypes, "ManagedEntity")):
        return self.core.getContentsRecursively(_type=type, root=self)

    def delegate(self, name):
        log.debug("Delegating %s" % name)
        return TaskDelegate(self, name)















import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def ClusterDasFdmHostState(vim, *args, **kwargs):
    '''The ClusterDasFdmHostState data object describes the availability state of each
    active host in a vSphere HA enabled cluster.In a vSphere HA cluster, the active
    hosts form a fault domain. A host is inactive if it is in standby or
    maintenance mode, or it has been disconnected from vCenter Server. A vSphere HA
    agent, called the Fault Domain Manager (FDM), runs on each host in the fault
    domain.One FDM serves as the master and the remaining FDMs as its slaves. The
    master is responsible for monitoring the availability of the hosts and VMs in
    the cluster, and restarting any VMs that fail due to a host failure or non-
    user-initiated power offs. The master is also responsible for reporting fault-
    domain state to vCenter Server.The master FDM is determined through election by
    the FDMs that are alive at the time. An election occurs in the following
    circumstances:The slaves are responsible for reporting state updates to the
    master and restarting VMs as required. All FDMs provide the VM/Application
    Health Monitoring Service.'''

    obj = vim.client.factory.create('{urn:vim25}ClusterDasFdmHostState')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 1:
        raise IndexError('Expected at least 2 arguments got: %d' % len(args))

    required = [ 'state' ]
    optional = [ 'stateReporter', 'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

ClusterProfileServiceType = Enum(
    'DPM',
    'DRS',
    'FT',
    'HA',

)




from pyvisdk.base.managed_object_types import ManagedObjectTypes

from pyvisdk.base.base_entity import BaseEntity

import logging

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

class DistributedVirtualSwitchManager(BaseEntity):
    '''The interface to provide relevant information about DistributedVirtualSwitch.'''

    def __init__(self, core, name=None, ref=None, type=ManagedObjectTypes.DistributedVirtualSwitchManager):
        super(DistributedVirtualSwitchManager, self).__init__(core, name=name, ref=ref, type=type)

    

    
    
    def QueryAvailableDvsSpec(self):
        '''This operation returns a list of switch product specifications that are
        supported by the vCenter Server.
        
        '''
        return self.delegate("QueryAvailableDvsSpec")()
    
    def QueryCompatibleHostForExistingDvs(self, container, recursive, dvs):
        '''This operation returns a list of hosts that are compatible with the given
        DistributedVirtualSwitch product specification.
        
        :param container: Where to look for hosts. Supported types of objects for this parameter are Datacenter, ComputeResource and Folder.
        
        :param recursive: Whether to search for hosts in the subfolders, if applicable. In the case when container is a Datacenter, the recursive flag is applied to its HostFolder.
        
        :param dvs: Search the host based on the specification published in the compatibleHostComponentProductInfo of a DistributedVirtualSwitch. If not set, it is assumed to be the specification that a DistributedVirtualSwitch would have if it is created with the default DistributedVirtualSwitchProductSpec.
        
        '''
        return self.delegate("QueryCompatibleHostForExistingDvs")(container, recursive, dvs)
    
    def QueryCompatibleHostForNewDvs(self, container, recursive, switchProductSpec=None):
        '''This operation returns a list of hosts that are compatible with the given
        DistributedVirtualSwitch product specification.
        
        :param container: Where to look for hosts. Supported types of objects for this parameter are Datacenter, ComputeResource and Folder.
        
        :param recursive: Whether to search for hosts in the subfolders, if applicable. In the case when container is a Datacenter, the recursive flag is applied to its HostFolder.
        
        :param switchProductSpec: The productSpec of a DistributedVirtualSwitch. If not set, it is assumed to be the default one used for DistributedVirtualSwitch creation.
        
        '''
        return self.delegate("QueryCompatibleHostForNewDvs")(container, recursive, switchProductSpec)
    
    def QueryDvsByUuid(self, uuid):
        '''This operation returns a DistributedVirtualSwitch given a UUID.
        
        :param uuid: 
        
        '''
        return self.delegate("QueryDvsByUuid")(uuid)
    
    def QueryDvsCheckCompatibility(self, hostContainer, dvsProductSpec=None, hostFilterSpec=None):
        '''This operation returns a list of compatibility results. Each compatibility
        result is an object that has a host property and optionally a fault which would
        be populated only if that host is not compatible with a given dvsProductSpec.
        All filters in hostFilerSpecs are ANDed to derive the intersection of hosts
        against which compatibility is checked. If caller did not have view privileges
        on the host entity in an element of the CompatibilityResult array, then that
        entire element would be removed from the CompatibilityResult array. Typical
        uses:This operation returns a list of compatibility results. Each compatibility
        result is an object that has a host property and optionally a fault which would
        be populated only if that host is not compatible with a given dvsProductSpec.
        All filters in hostFilerSpecs are ANDed to derive the intersection of hosts
        against which compatibility is checked. If caller did not have view privileges
        on the host entity in an element of the CompatibilityResult array, then that
        entire element would be removed from the CompatibilityResult array. Typical
        uses:
        
        :param hostContainer: The container of hosts on which we check the compatibility. This container can be a datacenter, folder, or computeResource. We can also include all the hosts in the hierarchy with container as root of the tree.
        
        :param dvsProductSpec: The productSpec of a DistributedVirtualSwitch. If not set, it is assumed to be the default one used for DistributedVirtualSwitch creation for current version.
        
        :param hostFilterSpec: The hosts against which to check compatibility. This is a filterSpec and users can use this to specify all hosts in a container (datacenter, folder, or computeResource), an array of hosts, or hosts that might or might not be a DVS member.
        
        '''
        return self.delegate("QueryDvsCheckCompatibility")(hostContainer, dvsProductSpec, hostFilterSpec)
    
    def QueryDvsCompatibleHostSpec(self, switchProductSpec=None):
        '''This operation returns a list of host product specifications that are
        compatible with the given DistributedVirtualSwitch product specification.
        
        :param switchProductSpec: The productSpec of a DistributedVirtualSwitch. If not set, it is assumed to be the default one used for DistributedVirtualSwitch creation.
        
        '''
        return self.delegate("QueryDvsCompatibleHostSpec")(switchProductSpec)
    
    def QueryDvsConfigTarget(self, host=None, dvs=None):
        '''This operation returns the DistributedVirtualSwitch or
        DistributedVirtualPortgroup config target on a host.
        
        :param host: The host on which the query is to be made. If called directly on the host this parameter need not be specified.
        
        :param dvs: The distributed virtual switch on which the query is to be made. If unspecified the config target will encompass all the distributed virtual switches available on the host.
        
        '''
        return self.delegate("QueryDvsConfigTarget")(host, dvs)
    
    def QueryDvsFeatureCapability(self, switchProductSpec=None):
        '''This operation indicates which version-specific DVS features are available for
        the given DistributedVirtualSwitch product specification.
        
        :param switchProductSpec: The productSpec of a DistributedVirtualSwitch. If not set, it is assumed to be the default one used for DistributedVirtualSwitch creation.
        
        '''
        return self.delegate("QueryDvsFeatureCapability")(switchProductSpec)
    
    def RectifyDvsOnHost_Task(self, hosts):
        '''Update the Distributed Switch configuration on the hosts to bring them in sync
        with the current configuration in vCenter Server.
        
        :param hosts: The hosts to be rectified.
        
        '''
        return self.delegate("RectifyDvsOnHost_Task")(hosts)



from pyvisdk.base.managed_object_types import ManagedObjectTypes

from pyvisdk.base.base_entity import BaseEntity

import logging

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

class LocalizationManager(BaseEntity):
    '''This managed object type presents all the message catalogs for client-side
    localization of messages.'''

    def __init__(self, core, name=None, ref=None, type=ManagedObjectTypes.LocalizationManager):
        super(LocalizationManager, self).__init__(core, name=name, ref=ref, type=type)

    
    @property
    def catalog(self):
        '''Fetches the descriptions of all the client-side localization message catalogs
        available for the current session locale.'''
        return self.update('catalog')

    



########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

HostUnresolvedVmfsExtentUnresolvedReason = Enum(
    'diskIdMismatch',
    'uuidConflict',

)




from pyvisdk.base.managed_object_types import ManagedObjectTypes

from pyvisdk.base.base_entity import BaseEntity

import logging

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

class ViewManager(BaseEntity):
    '''The ViewManager managed object provides methods to create ContainerView,
    InventoryView, and ListView managed objects. The ViewManager also maintains a
    list of managed object references to the views that you have created. Use the
    viewList property to access the views.A View is a mechanism that supports
    selection of objects on the server and subsequently, access to those objects.
    Views can simplify the task of retrieving data from the server. When you use a
    view, you can use a single invocation of a PropertyCollector method to retrieve
    data or receive notification of changes instead of multiple invocations
    involving multiple filter specifications. A view exists until you destroy it or
    until the end of the session.The ViewManager supports the following views:* A
    ContainerView is based on Folder, Datacenter, ComputeResource, ResourcePool, or
    HostSystem managed objects. Use a container view to monitor the container
    contents and optionally, its descendants. * A ListView managed object is based
    on an arbitrary but specific set of objects. When you create a list view, you
    provide a list of objects to populate the view (CreateListView), or you provide
    an existing view from which the new view is created (CreateListViewFromView). *
    An InventoryView managed object is based on the entire inventory. Use an
    inventory view as a general mechanism to monitor the inventory or portions of
    the inventory.For example, you might use the following sequence of operations
    to get the names of all the virtual machines on a server:'''

    def __init__(self, core, name=None, ref=None, type=ManagedObjectTypes.ViewManager):
        super(ViewManager, self).__init__(core, name=name, ref=ref, type=type)

    
    @property
    def viewList(self):
        '''An array of view references. Each array entry is a managed object reference to
        a view created by this ViewManager.'''
        return self.update('viewList')

    
    
    def CreateContainerView(self, container, recursive, type=None):
        '''Create a ContainerView managed object for this session. The method returns a
        reference to a ContainerView object that has a list of managed object
        references. The list references objects in the container and may include
        references to objects from additional containers. You can configure the
        resulting list of objects by specifying a type list and recursion. Once you
        have created the view, the object list always represents the current
        configuration of the virtual environment and reflects any subsequent changes
        that occur.
        
        :param container: A reference to an instance of a Folder, Datacenter, ComputeResource, ResourcePool, or HostSystem object.
        
        :param type: An optional list of managed entity types. The server associates only objects of the specified type(s) with the view. If you specify an empty array, the server uses all types.
        
        :param recursive: Whether to include only the immediate children of the container instance, or to include additional objects by following paths beyond the immediate children.* Folder object - childEntity property. If recursive is false, the container list includes the reference to the child entity in the folder instance. If recursive is true, the server will follow the child folder path(s) to collect additional childEntity references. * ResourcePool object - vm and resourcePool properties. If recursive is false, the object list will contain references to the virtual machines associated with this resource pool, and references to virtual machines associated with the immediate child resource pools. If recursive is true, the server will follow all child resource pool paths extending from the immediate children (and their children, and so on) to collect additional references to virtual machines. * ComputeResource object - host and resourcePool properties. If recursive is false, the object list will contain references to the host systems associated with this compute resource, references to virtual machines associated with the host systems, and references to virtual machines associated with the immediate child resource pools. If recursive is true, the server will follow the child resource pool paths (and their child resource pool paths, and so on) to collect additional references to virtual machines. * Datacenter object - vmFolder, hostFolder, datastoreFolder, and networkFolder properties. If recursive is set to false, the server uses the immediate child folders for the virtual machines, hosts, datastores, and networks associated with this datacenter. If recursive is set to true, the server will follow the folder paths to collect references to additional objects. * HostSystem object - vm property. The view object list contains references to the virtual machines associated with this host system. The value of recursive does not affect this behavior.
        
        '''
        return self.delegate("CreateContainerView")(container, type, recursive)
    
    def CreateInventoryView(self):
        '''Create a new InventoryView managed object for this session.
        
        '''
        return self.delegate("CreateInventoryView")()
    
    def CreateListView(self, obj=None):
        '''Create a ListView object for this session. The method returns a session object
        that has a list of managed object references. The list of references
        corresponds to the input object list. You can modify the resulting list after
        you have created the object.
        
        :param obj: The initial list of objects in the view.
        
        '''
        return self.delegate("CreateListView")(obj)
    
    def CreateListViewFromView(self, view):
        '''Create a ListView object for this session. This method uses an existing view to
        construct the object list for the new view.
        
        :param view: The view that will provide the object list for the new ListView object.
        
        '''
        return self.delegate("CreateListViewFromView")(view)



from pyvisdk.esxcli.executer import execute_soap
from pyvisdk.esxcli.base import Base

class SystemSettingsKeyboardLayout(Base):
    '''
    Operations that relate to keyboard layout
    '''
    moid = 'ha-cli-handler-system-settings-keyboard-layout'
    def set(self, layout=None, nopersist=None):
        '''
        Set the keyboard layout
        :param layout: string, The name of the layout to set
        :param nopersist: boolean, Only apply this layout for the current boot
        :returns: void
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.system.settings.keyboard.layout.Set',
                            layout=layout,
                            nopersist=nopersist,
                            )
    def list(self):
        '''
        List the keyboard layout
        :returns: vim.EsxCLI.system.settings.keyboard.layout.list.Layout[]
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.system.settings.keyboard.layout.List',
                            )
    def get(self):
        '''
        Get the keyboard layout
        :returns: string
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.system.settings.keyboard.layout.Get',
                            )   



from pyvisdk.esxcli.executer import execute_soap
from pyvisdk.esxcli.base import Base

class SoftwareAcceptance(Base):
    '''
    Retrieve and set the host acceptance level setting
    '''
    moid = 'ha-cli-handler-software-acceptance'
    def set(self, level):
        '''
        Sets the host acceptance level. This controls what VIBs will be allowed on a host.
        :param level: string, Specifies the acceptance level to set. Should be one of VMwareCertified / VMwareAccepted / PartnerSupported / CommunitySupported.
        :returns: string
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.software.acceptance.Set',
                            level=level,
                            )
    def get(self):
        '''
        Gets the host acceptance level. This controls what VIBs will be allowed on a host.
        :returns: string
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.software.acceptance.Get',
                            )   



import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def TaskEvent(vim, *args, **kwargs):
    '''This event records the creation of a Task. Note that the embedded TaskInfo
    object is a of the Task state at the time of its creation, so its state will
    always be "queued". To find the current status of the task, query for the
    current state of the Task using the eventChainId in the embedded TaskInfo
    object.'''

    obj = vim.client.factory.create('{urn:vim25}TaskEvent')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 5:
        raise IndexError('Expected at least 6 arguments got: %d' % len(args))

    required = [ 'info', 'chainId', 'createdTime', 'key', 'userName' ]
    optional = [ 'changeTag', 'computeResource', 'datacenter', 'ds', 'dvs',
        'fullFormattedMessage', 'host', 'net', 'vm', 'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




import logging
from pyvisdk.exceptions import InvalidArgumentError

# This module is NOT auto-generated
# Inspired by decompiled Java classes from vCenter's internalvim25stubs.jar
# Unless states otherside, the methods and attributes were not used by esxcli,
# and thus not tested

log = logging.getLogger(__name__)

def VimEsxCLIiscsinetworkportallistNetworkPortal(vim, *args, **kwargs):
    obj = vim.client.factory.create('{urn:vim25}VimEsxCLIiscsinetworkportallistNetworkPortal')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 0:
        raise IndexError('Expected at least 1 arguments got: %d' % len(args))

    required = [  ]
    optional = [ 'Adapter', 'CompliantStatus', 'CurrentSpeed', 'Duplex', 'IPv4', 'IPv4SubnetMask', 'IPv6', 'LinkUp', 'MACAddress', 'MACAddressValid', 'MTU', 'NICDriver', 'NICDriverVersion', 'NICFirmwareVersion', 'NonCompliantMessage', 'NonCompliantRemedy', 'PathStatus', 'PortGroup', 'PortGroupKey', 'PortKey', 'ReservedPorts', 'RxPackets', 'TCPChecksum', 'TOE', 'TSO', 'TxPackets', 'VlanID', 'VlanSupported', 'Vmknic', 'Vswitch', 'VswitchUuid' ]

    for name, arg in zip(required + optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj



import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def AlarmTriggeringActionTransitionSpec(vim, *args, **kwargs):
    '''Specification indicating which on transitions this action fires. The existence
    of a Spec indicates that this action fires on transitions from that Spec's
    startState to finalState.There are only four acceptable {startState,
    finalState} pairs: {green, yellow}, {yellow, red}, {red, yellow} and {yellow,
    green}. At least one of these pairs must be specified. Any deviation from the
    above will render the enclosing AlarmSpec invalid.'''

    obj = vim.client.factory.create('{urn:vim25}AlarmTriggeringActionTransitionSpec')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 3:
        raise IndexError('Expected at least 4 arguments got: %d' % len(args))

    required = [ 'finalState', 'repeats', 'startState' ]
    optional = [ 'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def VAppEntityConfigInfo(vim, *args, **kwargs):
    '''This object type describes the behavior of an entity (virtual machine or sub-
    vApp container) in a vApp container.The auto-start/auto-stop configurations
    control the behavior of the start/stop vApp operations.An virtual machine
    entity can be configured to wait for a period of time before starting or to
    wait to receive a successful heartbeat from a virtual machine before starting
    the next virtual machine in the sequence.If startAction and stopAction for an
    entity are both set to none, that entity does not participate in the
    sequence.The start/stop delay and waitingForGuest is not used if the entity is
    a vApp container. For a vApp the only value values for startAction is none or
    powerOn, and the valid values for stopAction is none or powerOff.'''

    obj = vim.client.factory.create('{urn:vim25}VAppEntityConfigInfo')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 0:
        raise IndexError('Expected at least 1 arguments got: %d' % len(args))

    required = [  ]
    optional = [ 'destroyWithParent', 'key', 'startAction', 'startDelay', 'startOrder',
        'stopAction', 'stopDelay', 'tag', 'waitingForGuest', 'dynamicProperty',
        'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def ResourcePoolQuickStats(vim, *args, **kwargs):
    '''A set of statistics that are typically updated with near real-time regularity.
    These statistics are aggregates of the corresponding statistics of all virtual
    machines in the given resource pool, and unless otherwise noted, only make
    sense when at least one virtual machine in the given resource pool is powered
    on. This data object type does not support notification, for scalability
    reasons. Therefore, changes in QuickStats do not generate property collector
    updates. To monitor statistics values, use the statistics and alarms modules
    instead.'''

    obj = vim.client.factory.create('{urn:vim25}ResourcePoolQuickStats')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 0:
        raise IndexError('Expected at least 1 arguments got: %d' % len(args))

    required = [  ]
    optional = [ 'balloonedMemory', 'compressedMemory', 'consumedOverheadMemory',
        'distributedCpuEntitlement', 'distributedMemoryEntitlement',
        'guestMemoryUsage', 'hostMemoryUsage', 'overallCpuDemand', 'overallCpuUsage',
        'overheadMemory', 'privateMemory', 'sharedMemory', 'staticCpuEntitlement',
        'staticMemoryEntitlement', 'swappedMemory', 'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




from pyvisdk.esxcli.executer import execute_soap
from pyvisdk.esxcli.base import Base

class IscsiLogicalnetworkportal(Base):
    '''
    Operations that can be performed on iSCSI Logical Network Portal (vmknic)
    '''
    moid = 'ha-cli-handler-iscsi-logicalnetworkportal'
    def list(self, adapter=None):
        '''
        List Logical Network Portals for iSCSI Adapter
        :param adapter: string, The iSCSI adapter name.
        :returns: vim.EsxCLI.iscsi.logicalnetworkportal.list.LogicalNetworkPortal[]
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.iscsi.logicalnetworkportal.List',
                            adapter=adapter,
                            )   



import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def Event(vim, *args, **kwargs):
    '''This event is the base data object type from which all events inherit. All
    event objects are data structures that describe events. While event data
    objects are data structures that describe events, event data type documentation
    may describe what the event records, rather than the data structure, itself.'''

    obj = vim.client.factory.create('{urn:vim25}Event')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 4:
        raise IndexError('Expected at least 5 arguments got: %d' % len(args))

    required = [ 'chainId', 'createdTime', 'key', 'userName' ]
    optional = [ 'changeTag', 'computeResource', 'datacenter', 'ds', 'dvs',
        'fullFormattedMessage', 'host', 'net', 'vm', 'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




from pyvisdk.esxcli.executer import execute_soap
from pyvisdk.esxcli.base import Base

class NetworkIpInterfaceIpv6Address(Base):
    '''
    Commands to list and update IPv6 addresses assigned to the system.
    '''
    moid = 'ha-cli-handler-network-ip-interface-ipv6-address'
    def add(self, interfacename, ipv6):
        '''
        Add a static IPv6 address to a given VMkernel network interface.
        :param interfacename: string, The name of the VMkernel network interface to add a static IPv6 address to. This name must be an interface listed in the interface list command.
        :param ipv6: string, The IPv6 address to add to the given VMkernel network interface. This must be in X:X:X::/X format
        :returns: boolean
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.network.ip.interface.ipv6.address.Add',
                            interfacename=interfacename,
                            ipv6=ipv6,
                            )
    def list(self):
        '''
        This command will list all of the IPv6 addresses currently assigned to the system
        :returns: vim.EsxCLI.network.ip.interface.ipv6.address.list.IPv6Interface[]
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.network.ip.interface.ipv6.address.List',
                            )
    def remove(self, interfacename, ipv6):
        '''
        Remove an IPv6 address from a given VMkernel network interface.
        :param interfacename: string, The name of the VMkernel network interface to remove an IPv6 address from. This name must be an interface listed in the interface list command.
        :param ipv6: string, The IPv6 address to remove from the given VMkernel network interface. This must be in X:X:X::/X format
        :returns: boolean
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.network.ip.interface.ipv6.address.Remove',
                            interfacename=interfacename,
                            ipv6=ipv6,
                            )   



########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

DatastoreSummaryMaintenanceModeState = Enum(
    'enteringMaintenance',
    'inMaintenance',
    'normal',

)




from pyvisdk.esxcli.executer import execute_soap
from pyvisdk.esxcli.base import Base

class StorageCoreAdapter(Base):
    '''
    Operations on SCSI Host Bus Adapters on the system.
    '''
    moid = 'ha-cli-handler-storage-core-adapter'
    def rescan(self, adapter=None, all=None, skipclaim=None, skipfsscan=None, type=None):
        '''
        Rescan SCSI HBAs to search for new Devices, remove DEAD paths and update path state. This operation will also run an claim operation equivalent to the claimrule run command and a filesystem rescan.
        :param adapter: string, Select the adapter to use when rescanning SCSI adapters. This must be a SCSI HBA name as shown in the adapter list command. This cannot be used with the --all option
        :param all: boolean, Indicate the rescan should rescan all adapters instead of a specific one.
        :param skipclaim: boolean, By default after an add operation a claiming session is run to find new devices and have them be claimed by the appropriate Multipath Plugin. Passing this flag will skip that claiming session.
        :param skipfsscan: boolean, By default after all rescan operations a filesystem scan is performed to add newly found filesystems and remove any filesystems that are no longer available. Passing this flag will skip that filesystem scan.
        :param type: string, Specify the type of rescan to perform. Available types are 
    add: Perform rescan and only add new devices if any.
    all: Perform rescan and do all opertaions (this is the default action.)
    delete: Perform rescan and only delete DEAD devices.
    update: Rescan existing paths only and update path states.

        :returns: void
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.storage.core.adapter.Rescan',
                            adapter=adapter,
                            all=all,
                            skipclaim=skipclaim,
                            skipfsscan=skipfsscan,
                            type=type,
                            )
    def list(self):
        '''
        List all the SCSI Host Bus Adapters on the system.
        :returns: vim.EsxCLI.storage.core.adapter.list.ScsiAdapter[]
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.storage.core.adapter.List',
                            )   



import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def DistributedVirtualSwitchManagerHostContainer(vim, *args, **kwargs):
    '''Check host compatibility for all hosts in the container. If the recursive flag
    is true, then check hosts at all levels within this container, otherwise check
    only at the container level. In case of container being a Datacenter, the
    recursive flag is applied to its HostFolder.'''

    obj = vim.client.factory.create('{urn:vim25}DistributedVirtualSwitchManagerHostContainer')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 2:
        raise IndexError('Expected at least 3 arguments got: %d' % len(args))

    required = [ 'container', 'recursive' ]
    optional = [ 'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




from pyvisdk.esxcli.executer import execute_soap
from pyvisdk.esxcli.base import Base

class SystemTime(Base):
    '''
    Commands to get and set system time.
    '''
    moid = 'ha-cli-handler-system-time'
    def set(self, day=None, hour=None, min=None, month=None, sec=None, year=None):
        '''
        Set the system clock time. Any missing parameters will default to the current time
        :param day: long, Day
        :param hour: long, Hour
        :param min: long, Minute
        :param month: long, Month
        :param sec: long, Second
        :param year: long, Year
        :returns: boolean
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.system.time.Set',
                            day=day,
                            hour=hour,
                            min=min,
                            month=month,
                            sec=sec,
                            year=year,
                            )
    def get(self):
        '''
        Disply the current system time.
        :returns: string
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.system.time.Get',
                            )   



from pyvisdk.esxcli.executer import execute_soap
from pyvisdk.esxcli.base import Base

class SystemCoredumpPartition(Base):
    '''
    Operations pertaining to the VMkernel Core dump configuration on VMkernel Diagnostic partitions. 
    '''
    moid = 'ha-cli-handler-system-coredump-partition'
    def set(self, enable=None, partition=None, smart=None, unconfigure=None):
        '''
        Set the specific VMkernel dump partition for this system. This will configure the dump partition for the next boot. This command will change the active dump partition to the partition specified.
        :param enable: boolean, Enable or disable the VMkernel dump partition. This option cannot be specified when setting or unconfiguring the dump partition.
        :param partition: string, The name of the partition to use. This should be a device name with a partition number at the end. Example: naa.xxxxx:1
        :param smart: boolean, This flag can be used only with --enable=true. It will cause the best available partition to be selected using the smart selection algorithm.
        :param unconfigure: boolean, Set the dump partition into an unconfigured state. This will remove the current configured dump partition for the next boot. This will result in the smart activate algorithm being used at the next boot.
        :returns: boolean
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.system.coredump.partition.Set',
                            enable=enable,
                            partition=partition,
                            smart=smart,
                            unconfigure=unconfigure,
                            )
    def list(self):
        '''
        List all of the partitions on the system that have a partition type matching the VMware Core partition type. Also indicate which partition, if any, is being used as the system's dump partition and which is configured to be used at next boot.
        :returns: vim.EsxCLI.system.coredump.partition.list.DumpPartition[]
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.system.coredump.partition.List',
                            )
    def get(self):
        '''
        Get one of the dump partition configured values. This command will print either the active dump partition or the configured dump partition depending on the flags passed.
        :returns: vim.EsxCLI.system.coredump.partition.get.KernelDumpParition
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.system.coredump.partition.Get',
                            )   



import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def PerfInterval(vim, *args, **kwargs):
    '''This data object type contains metadata about a performance interval.For ESX
    systems, a single instance of this data object exists. It cannot be modified.
    It has these properties:VirtualCenter Server system provides four instances of
    this data object by default, that apply globally to all system
    entities.VirtualCenter Server uses the specifications configured in its
    historical intervals to collect metrics from the ESX systems that it manages.
    The quantity of data collected depends on the level settings for the server,
    and the level associated with a specific counter. Both factors may change from
    one version of the products to the next. In general, the lower the number, the
    smaller the amount of data collected. For VirtualCenter Server 2.5, for
    example, the levels 1 through 4 collected data as follows:Default properties
    for the four built-in historical intervals include:All values are in seconds.
    The default setting for vCenter Server is level 1, which retains sampled
    statistical data as follows:Data older than a year is purged from the vCenter
    Server database.Prior to version 2.5 of the API, this data object could be used
    in conjunction with the CreatePerfInterval operation, to define new, custom
    historical intervals. That operation has been deprecated: Adding and deleting
    objects of this type is no longer supported. However, the default historical
    intervals can be enabled or disabled, and can be modified within certain limits
    (with the UpdatePerfInterval operation).'''

    obj = vim.client.factory.create('{urn:vim25}PerfInterval')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 5:
        raise IndexError('Expected at least 6 arguments got: %d' % len(args))

    required = [ 'enabled', 'key', 'length', 'name', 'samplingPeriod' ]
    optional = [ 'level', 'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




from pyvisdk.base.managed_object_types import ManagedObjectTypes

from pyvisdk.base.base_entity import BaseEntity

import logging

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

class ExtensibleManagedObject(BaseEntity):
    '''ExtensibleManagedObject provides methods and properties that provide access to
    custom fields that may be associated with a managed object. Use the
    CustomFieldsManager to define custom fields. The CustomFieldsManager handles
    the entire list of custom fields on a server. You can can specify the object
    type to which a particular custom field applies by setting its
    managedObjectType. (If you do not set a managed object type for a custom field
    definition, the field applies to all managed objects.)'''

    def __init__(self, core, name=None, ref=None, type=ManagedObjectTypes.ExtensibleManagedObject):
        super(ExtensibleManagedObject, self).__init__(core, name=name, ref=ref, type=type)

    
    @property
    def availableField(self):
        '''List of custom field definitions that are valid for the object's type. The
        fields are sorted by name.'''
        return self.update('availableField')
    @property
    def value(self):
        '''List of custom field values. Each value uses a key to associate an instance of
        a CustomFieldStringValue with a custom field definition.'''
        return self.update('value')

    
    
    def setCustomValue(self, key, value):
        '''Assigns a value to a custom field. The setCustomValue method requires whichever
        updatePrivilege is defined as one of the fieldInstancePrivileges for the
        CustomFieldDef whose value is being changed.
        
        :param key: The name of the field whose value is to be updated.
        
        :param value: Value to be assigned to the custom field.
        
        '''
        return self.delegate("setCustomValue")(key, value)



import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def NetworksMayNotBeTheSame(vim, *args, **kwargs):
    '''Used as a warning if a virtual machine provisioning operation is done across
    datacenters. This warns that the network used by the virtual machine before and
    after the operation may not be the same even though the two networks have the
    same name. This is because network names are only unique within a datacenter.'''

    obj = vim.client.factory.create('{urn:vim25}NetworksMayNotBeTheSame')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 5:
        raise IndexError('Expected at least 6 arguments got: %d' % len(args))

    required = [ 'name', 'dynamicProperty', 'dynamicType', 'faultCause', 'faultMessage' ]
    optional = [  ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

VAppIPAssignmentInfoAllocationSchemes = Enum(
    'dhcp',
    'ovfenv',

)




from pyvisdk.esxcli.executer import execute_soap
from pyvisdk.esxcli.base import Base

class SoftwareSourcesVib(Base):
    '''
    Query depot contents for VIBs; display information about VIB URLs and files
    '''
    moid = 'ha-cli-handler-software-sources-vib'
    def list(self, depot, proxy=None):
        '''
        List all the VIBs from depots.
        :param depot: string[], Specifies full remote URLs of the depot index.xml or server file path pointing to an offline bundle .zip file.
        :param proxy: string, Specifies a proxy server to use for HTTP, FTP, and HTTPS connections. The format is proxy-url:port.
        :returns: vim.EsxCLI.software.sources.vib.list.VIBSummary[]
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.software.sources.vib.List',
                            depot=depot,
                            proxy=proxy,
                            )
    def get(self, depot=None, proxy=None, vibname=None, viburl=None):
        '''
        Displays detailed information about one or more VIB packages in the depot
        :param depot: string[], Specifies full remote URLs of the depot index.xml or server file path pointing to an offline bundle .zip file.
        :param proxy: string, Specifies a proxy server to use for HTTP, FTP, and HTTPS connections. The format is proxy-url:port.
        :param vibname: string[], Specifies one or more VIBs in the depot to display more information about. If this option is not specified, then all of the VIB packages from the depot will be displayed. Must be one of the following forms: name, name:version, vendor:name, or vendor:name:version.
        :param viburl: string[], Specifies one or more URLs to VIB packages to display information about. http:, https:, ftp:, and file: are all supported.
        :returns: vim.EsxCLI.software.sources.vib.get.VIB[]
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.software.sources.vib.Get',
                            depot=depot,
                            proxy=proxy,
                            vibname=vibname,
                            viburl=viburl,
                            )   



########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

VirtualMachineMovePriority = Enum(
    'defaultPriority',
    'highPriority',
    'lowPriority',

)




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

MultipathState = Enum(
    'active',
    'dead',
    'disabled',
    'standby',
    'unknown',

)




from pyvisdk.base.managed_object_types import ManagedObjectTypes

from pyvisdk.base.base_entity import BaseEntity

import logging

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

class IscsiManager(BaseEntity):
    '''This managed object provides interfaces for mapping VMkernel NIC to iSCSI Host
    Bus Adapter.'''

    def __init__(self, core, name=None, ref=None, type=ManagedObjectTypes.IscsiManager):
        super(IscsiManager, self).__init__(core, name=name, ref=ref, type=type)

    

    
    
    def BindVnic(self, iScsiHbaName, vnicDevice):
        '''Bind a Virtual NIC to be used for an iSCSI adapter
        
        :param iScsiHbaName: iSCSI adapter name for which the Virtual NIC to be added.
        
        :param vnicDevice: Virtual NIC that is to be bound to the iSCSI HBA
        
        '''
        return self.delegate("BindVnic")(iScsiHbaName, vnicDevice)
    
    def QueryBoundVnics(self, iScsiHbaName):
        '''Query the list of Virtual NICs that are bound to a given iSCSI HBA.
        
        :param iScsiHbaName: iSCSI adapter name for which the method to be applied.
        
        '''
        return self.delegate("QueryBoundVnics")(iScsiHbaName)
    
    def QueryCandidateNics(self, iScsiHbaName):
        '''Query the candidate Virtual NICs and Physical NICs that can be used for Port-
        Binding. For dependent offload adapters, the Virtual NIC should be attached to
        the physical NIC associated with the hardware function.
        
        :param iScsiHbaName: iSCSI Adapter name for which the method to be applied.
        
        '''
        return self.delegate("QueryCandidateNics")(iScsiHbaName)
    
    def QueryMigrationDependencies(self, pnicDevice):
        '''Query the dependency table for a migration operation of a given Physical NIC.
        
        :param pnicDevice: List of Physical NICs to be migrated
        
        '''
        return self.delegate("QueryMigrationDependencies")(pnicDevice)
    
    def QueryPnicStatus(self, pnicDevice):
        '''Query if Physical NIC device is used for iSCSI.
        
        :param pnicDevice: Physical NIC device name to check the status for
        
        '''
        return self.delegate("QueryPnicStatus")(pnicDevice)
    
    def QueryVnicStatus(self, vnicDevice):
        '''Query the status of Virtual NIC association with the iSCSI.
        
        :param vnicDevice: Virtual NIC device to check the status for
        
        '''
        return self.delegate("QueryVnicStatus")(vnicDevice)
    
    def UnbindVnic(self, iScsiHbaName, vnicDevice, force):
        '''Unbind Virtual NIC binding from an iSCSI adapter.
        
        :param iScsiHbaName: iSCSI adapter name for which the Virtual NIC to be removed.
        
        :param vnicDevice: Virtual NIC that is to be removed from the iSCSI HBA
        
        :param force: 
        
        '''
        return self.delegate("UnbindVnic")(iScsiHbaName, vnicDevice, force)



from pyvisdk.esxcli.executer import execute_soap
from pyvisdk.esxcli.base import Base

class NetworkIpDnsServer(Base):
    '''
    Operations pertaining to DNS server configuration.
    '''
    moid = 'ha-cli-handler-network-ip-dns-server'
    def add(self, server):
        '''
        Add a new DNS server to the end of the list of DNS servers to use for this ESXi host.
        :param server: string, The IP address (v4 or v6) of the DNS server to add to the DNS server list.
        :returns: boolean
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.network.ip.dns.server.Add',
                            server=server,
                            )
    def list(self):
        '''
        Print a list of the DNS server currently configured on the system in the order in which they will be used.
        :returns: vim.EsxCLI.network.ip.dns.server.list.NameServerList
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.network.ip.dns.server.List',
                            )
    def remove(self, server, all=None):
        '''
        Remove a DNS server from the list of DNS servers to use for this ESXi host.
        :param all: boolean,  
        :param server: string,  
        :returns: boolean
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.network.ip.dns.server.Remove',
                            all=all,
                            server=server,
                            )   



########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

VirtualMachineUsbInfoFamily = Enum(
    'audio',
    'bluetooth',
    'communication',
    'hid',
    'hid_bootable',
    'hub',
    'imaging',
    'other',
    'pda',
    'physical',
    'printer',
    'security',
    'smart_card',
    'storage',
    'unknownFamily',
    'vendor_specific',
    'video',
    'wireless',
    'wusb',

)




from pyvisdk.esxcli.executer import execute_soap
from pyvisdk.esxcli.base import Base

class StorageCoreDeviceVaaiStatus(Base):
    '''
    Query a SCSI device's VAAI state.
    '''
    moid = 'ha-cli-handler-storage-core-device-vaai-status'
    def get(self, device=None):
        '''
        List VAAI properties for devices currently registered with the PSA.
        :param device: string, Filter the output of this command to only show a single device.
        :returns: vim.EsxCLI.storage.core.device.vaai.status.get.ScsiDevice[]
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.storage.core.device.vaai.status.Get',
                            device=device,
                            )   


from pyvisdk.base.managed_object_types import ManagedObjectTypes
from pyvisdk.do.managed_object_reference import ManagedObjectReference
from brownie.importing import import_string
from pyvisdk.utils import camel_to_under

import logging
import pyvisdk.core

#
# Assumptions:  Must connect to the vSphere vCenter
#               Must be version 4.0 or greater
#

log = logging.getLogger(__name__)

class VimMixin(object):
    def getReferenceToManagedObject(self, mo):
        """
        Returns reference of managed object as string. For example: VirtualMachine:vm-16
        """
        return "{}:{}".format(mo.ref._type, mo.ref.value)

    def _managedObjectExists(self, mo):
        from suds import WebFault
        try:
            mo.value    # test any member of managed object.. 'value' is common to all objects
        except WebFault, e:
            if e.fault.detail.__keylist__ == ['ManagedObjectNotFoundFault']:
                return False
        return True

    def getManagedObjectByReference(self, moref):
        """
        Returns the managed object
        :param moref: moref string. For example: VirtualMachine:vm-16
        :rtype: :py:class:`ManagedEntity`
        """
        # http://www.doublecloud.org/2011/03/how-to-get-a-managed-object-with-its-id-like-task-id/
        class_name, object_id = moref.split(':', 1)
        ref = ManagedObjectReference(self, type=class_name, value=object_id)
        mo = self._getManagedObjectType(class_name)(self, ref=ref)
        if not self._managedObjectExists(mo):
            raise ValueError(object_id)
        return mo

    def _getManagedObjectType(self, class_name):
        # inspired by pyvisdk.vim
        try:
            return import_string("pyvisdk.mo.{}.{}".format(camel_to_under(class_name), class_name))
        except ImportError:
            raise ValueError(class_name)


class Vim(VimMixin, pyvisdk.core.VimBase):
    def __init__(self, server, connect=True, verbose=3, certfile=None, keyfile=None, wait_for_task=True):
        super(Vim, self).__init__(server, connect, verbose, certfile, keyfile, wait_for_task=wait_for_task)
        self.loggedin = False
        self.username = None
        self.password = None
        self.facades = dict()

    def login(self, username, password, locale=None):
        """
        Log into the vmware server.
        """
        self.username = username
        self.password = password

        if not locale:
            if hasattr(self.session_manager, 'defaultLocale'):
                locale = self.session_manager.defaultLocale
            else:
                locale = 'en_US'

        if self.verbose > 5:
            self.displayAbout()

        self.session = self.session_manager.Login(self.username, self.password, locale)
        if self.verbose > 2:
            log.info("Successfully logged into %s" % self.client.url)
        self.loggedin = True
        self.facades = dict()

    def loginByExtensionCertificate(self, extension_key, locale=None):
        """
        Log into the vmware server with extension key
        """
        self.extension_key = extension_key
        if not locale:
            if hasattr(self.session_manager, 'defaultLocale'):
                locale = self.session_manager.defaultLocale
            else:
                locale = 'en_US'

        if self.verbose > 5:
            self.displayAbout()
        self.session = self.session_manager.LoginExtensionByCertificate(extension_key, locale)
        if self.verbose > 2:
            log.info("Successfully logged into %s" % self.proxy)
        self.username = self.session.userName
        self.loggedin = True
        self.facades = dict()

    def logout(self):
        """
        Log out of the vmware server.
        """
        self.session_manager.Logout()
        self.loggedin = False
        self.facades = dict()

    def relogin(self):
        """
        Re-logs in to the vmware server.
        """
        try:
            self.logout()
        except Exception:
            pass

        if self.username and self.password:
            self.login(self.username, self.password)

    def displayAbout(self):
        """
        Display version information about the vmware server and library
        """
        print "=" * 40
        print "Connected to %s" % self.server
        print "  %s" % self.service_content.about.fullName
        print "  OS: %s" % self.service_content.about.osType
        print "  License: %s %s" % (
                self.service_content.about.licenseProductName, self.service_content.about.licenseProductVersion)
        print "  Current Sessions: "
        for session in self.session_manager.sessionList:
            print "%-30s %-20s Last Active: %-20s Logged In: %-20s" % ("%s(%s)" % (session.fullName, session.userName), session.key, session.lastActiveType, session.loginTime)
        print "=" * 40

    def getApiType(self):
        """
        Get the API type

        :returns: Indicates whether or not the service instance represents a standalone host. If the service instance represents a standalone host, then the physical inventory for that service instance is fixed to that single host. VirtualCenter server provides additional features over single hosts. For example, VirtualCenter offers multi-host management.
        :rtype: xsd:string
        """
        return self.service_content.about.apiType

    #------------------------------------------------------------
    # Hosts
    #------------------------------------------------------------
    def getHostSystems(self):
        """
        Get all the hosts on the server

        :rtype: :py:class:`HostSystem`
        """
        return self.getDecendentsByName(_type=ManagedObjectTypes.HostSystem, properties=["name"]) #@UndefinedVariable

    def getHostSystem(self, _name=None):
        """
        Get the host system by name

        :rtype: :py:class:`HostSystem`
        """
        return self.getDecendentsByName(_type=ManagedObjectTypes.HostSystem, properties=["name"], name=_name) #@UndefinedVariable

    #------------------------------------------------------------
    # Datacenters
    #------------------------------------------------------------
    def getDatacenters(self):
        """
        Get all the data centers on the server

        :rtype: :py:class:`Datacenter`
        """
        return self.getDecendentsByName(_type=ManagedObjectTypes.Datacenter, properties=["name"]) #@UndefinedVariable

    def getDatacenter(self, _name):
        """
        Get the data center by name

        :rtype: :py:class:`Datacenter`
        """
        return self.getDecendentsByName(_type=ManagedObjectTypes.Datacenter, properties=["name"], name=_name) #@UndefinedVariable

    #------------------------------------------------------------
    # Datastores
    #------------------------------------------------------------
    def getDatastores(self):
        """
        Get all the data centers on the server

        :rtype: :py:class:`Datastore`
        """
        datastores = list()
        for datacenter in self.getDatacenters():
            datastores.extend(datacenter.datastore)
        return datastores

    def getDatastore(self, _name):
        """
        Get the data center by name

        :rtype: :py:class:`Datastore`
        """
        for datacenter in self.getDatacenters():
            for datastore in datacenter.datastore:
                if datastore.name == _name:
                    return datastore
        return None

    #------------------------------------------------------------
    # Resource pool
    #------------------------------------------------------------
    def getResourcePools(self):
        return self.getDecendentsByName(_type=ManagedObjectTypes.ResourcePool, properties=["name"]) #@UndefinedVariable

    def getResourcePool(self, _name):
        return self.getDecendentsByName(_type=ManagedObjectTypes.ResourcePool, properties=["name"], name=_name) #@UndefinedVariable

    #------------------------------------------------------------
    # Virtual Machines
    #------------------------------------------------------------
    def getVirtualMachine(self, _name):
        """
        Get the virtual machine by name

        :rtype: :py:class:`VirtualMachine`
        """
        return self.getDecendentsByName(_type=ManagedObjectTypes.VirtualMachine, properties=["name", "runtime.powerState"], name=_name) #@UndefinedVariable

    def getVirtualMachines(self):
        """
        Get all the virtual machines on the server

        :rtype: :py:class:`VirtualMachine`
        """
        return self.getDecendentsByName(_type=ManagedObjectTypes.VirtualMachine, properties=["name", "runtime.powerState"]) #@UndefinedVariable

    def getVirtualApps(self):
        return self.getDecendentsByName(_type=ManagedObjectTypes.VirtualApp, properties=["name"]) #@UndefinedVariable

    def getFolders(self):
        return self.getDecendentsByName(_type=ManagedObjectTypes.Folder, properties=["name"]) #@UndefinedVariable

    #------------------------------------------------------------
    # Hierarchy
    #------------------------------------------------------------
    def _getHierarchy(self):
        mo = self.getContentsRecursively(props=["configIssue", "configStatus", "name", "parent"])
        return mo


class Sms(VimMixin, pyvisdk.core.SmsBase):
    def __init__(self, vim, connect=True, wait_for_task=True):
        super(Sms, self).__init__(vim, wait_for_task=wait_for_task)
        if connect:
            self.connect()




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

HostInternetScsiHbaDigestType = Enum(
    'digestDiscouraged',
    'digestPreferred',
    'digestProhibited',
    'digestRequired',

)




import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def ResourcePoolResourceUsage(vim, *args, **kwargs):
    '''Specifies the resource usage for either memory or CPU. For CPU the unit is MHz,
    for memory the unit is bytes.In the typical case, where a resourcepool is in a
    consistent state, unreservedForVm will be equal to unreservedForPool. Hence, we
    can simply say talk about unreserved resources.If the reservation on the
    resource pool is not expandable, then the following is true:If the reservation
    on the resource pool is expandable, then the following is true:'''

    obj = vim.client.factory.create('{urn:vim25}ResourcePoolResourceUsage')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 6:
        raise IndexError('Expected at least 7 arguments got: %d' % len(args))

    required = [ 'maxUsage', 'overallUsage', 'reservationUsed', 'reservationUsedForVm',
        'unreservedForPool', 'unreservedForVm' ]
    optional = [ 'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def VirtualSerialPortDeviceBackingInfo(vim, *args, **kwargs):
    '''The data object defines information for using a host serial port device as
    backing for a . On a host, the first virtual machine to configure physical
    device backing for a virtual serial port will obtain the mapping. As long as
    that machine maintains the backing, any additional attempts to configure
    backing using that device will fail (a recoverable error, see the connection
    info ).'''

    obj = vim.client.factory.create('{urn:vim25}VirtualSerialPortDeviceBackingInfo')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 1:
        raise IndexError('Expected at least 2 arguments got: %d' % len(args))

    required = [ 'deviceName' ]
    optional = [ 'useAutoDetect', 'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def OnceTaskScheduler(vim, *args, **kwargs):
    '''The OnceTaskScheduler data object establishes the time for running a scheduled
    task only once.'''

    obj = vim.client.factory.create('{urn:vim25}OnceTaskScheduler')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 0:
        raise IndexError('Expected at least 1 arguments got: %d' % len(args))

    required = [  ]
    optional = [ 'runAt', 'activeTime', 'expireTime', 'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def WeeklyTaskScheduler(vim, *args, **kwargs):
    '''The WeeklyTaskScheduler data object sets the time for weekly task execution.
    You can set the schedule for task execution on one or more days during the
    week, and you complete the schedule by setting the inherited properties for the
    hour and minute.By default the scheduler executes the task according to the
    specified day(s) every week. If you set the interval to a value greater than 1,
    the task will execute at the specified weekly interval. (For example, an
    interval of 2 will cause the task to execute on the specified days every 2
    weeks.)'''

    obj = vim.client.factory.create('{urn:vim25}WeeklyTaskScheduler')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 10:
        raise IndexError('Expected at least 11 arguments got: %d' % len(args))

    required = [ 'friday', 'monday', 'saturday', 'sunday', 'thursday', 'tuesday', 'wednesday',
        'hour', 'minute', 'interval' ]
    optional = [ 'activeTime', 'expireTime', 'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




from pyvisdk.base.managed_object_types import ManagedObjectTypes

from pyvisdk.mo.managed_entity import ManagedEntity

import logging

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

class ResourcePool(ManagedEntity):
    '''Represents a set of physical resources: a single host, a subset of a host's
    resources, or resources spanning multiple hosts. Resource pools can be
    subdivided by creating child resource pools. In order to run, a virtual machine
    must be associated as a child of a resource pool.In a parent/child hierarchy of
    resource pools and virtual machines, the single resource pool that has no
    parent pool is known as the .ConfigurationA resource pool is configured with a
    set of CPU (in MHz) and memory (in MB) resources. These resources are specified
    in absolute terms with a resource reservation and a resource limit, along with
    a shares setting. The shares are used during resource contention, to ensure
    graceful degradation.For the root resource pool, the values of the reservation
    and the limit are set by the system and are not configurable. The reservation
    and limit are set to the same value, indicating the total amount of resources
    the system has available to run virtual machines. This is computed as the
    aggregated CPU and memory resources provided by the set of current available
    hosts in the parent compute resource minus the overhead of the virtualization
    layer.Since the resource pool configuration is absolute (in MHz or MB), the
    configuration can become invalid when resources are removed. This can happen if
    a host is removed from the cluster, if a host becomes unavailable, or if a host
    is placed in maintenance mode. When this happens, the system flags
    misconfigured resource pools and displays the reservations and limits that are
    in effect. Further, in a DRS enabled cluster, the tree can be misconfigured if
    the user bypasses VirtualCenter and powers on VMs directly on the host.There
    are three states that the resource pool tree can be in: undercommited (green),
    overcommited (yellow), and inconsistent (red). Depending on the state,
    different resource pool configuration policies are enforced. The states are
    described in more detail below:* GREEN (aka undercommitted): We have a tree
    that is in a state. Every node has a reservation greater than the sum of the
    reservations for its children. We have enough capacity at the root to satisfy
    all the resources reserved by the children. All operations performed on the
    tree, such as powering on virtual machines, creating new resource pools, or
    reconfiguring resource settings, will ensure that the above constraints are
    maintained. * RED (aka. inconsistent): One or more nodes in the tree has
    children whose reservations are greater than the node is configured to support.
    For example, i) a resource pool with a fixed reservation has a running virtual
    machine with a reservation that is higher than the reservation on resource pool
    itself., or ii) the child reservations are greater than the limit.In this
    state, the DRS algorithm is disabled until the resource pool tree's
    configuration has been brought back into a consistent state. We also restrict
    the resources that such invalid nodes request from their parents to the
    configured reservation/limit, in an attempt to isolate the problem to a small
    subtree. For the rest of the tree, we determine whether the cluster is
    undercommitted or overcommitted according to the existing rules and perform
    admission control accordingly.Note that since all changes to the resource
    settings are validated on the VirtualCenter server, the system cannot be
    brought into this state by simply manipulating a cluster resource pool tree
    through VirtualCenter. It can only happen if a virtual machine gets powered on
    directly on a host that is part of a DRS cluster.* YELLOW (aka overcommitted):
    In this state, the tree is consistent internally, but the root resource pool
    does not have the capacity at to meet the reservation of its children. We can
    only go from GREEN -> YELLOW if we lose resources at the root. For example,
    hosts becomes unavailable or is put into maintenance mode. Note that we will
    always have enough capacity at the root to run all currently powered on VMs.
    However, we may not be able to satisfy all resource pool reservations in the
    tree. In this state, the reservation configured for a resource pool is no
    longer guaranteed, but the limits are still enforced. This provides additional
    flexibility for bringing the tree back into a consistent state, without risking
    bringing the tree into a RED state. In more detail: * Resource Pool The root is
    considered to have unlimited capacity. You can reserve resources without any
    check except the requirement that the tree remains consistent. This means that
    nodes whose parents are all configured with expandable reservations and no
    limit will have unlimited available resources. However, if there is an ancestor
    with a fixed reservation or an expandable reservation with a limit somewhere,
    then the node will be limited by the reservation/limit of the ancestor. *
    Virtual Machine Virtual machines are limited by ancestors with a fixed
    reservation and the capacity at the root.Destroying a ResourcePoolWhen a
    ResourcePool is destroyed, all the virtual machines are reassigned to its
    parent pool. The root resource pool cannot be destroyed, and invoking destroy
    on it will throw an InvalidType fault.Any vApps in the ResourcePool will be
    moved to the ResourcePool's parent before the pool is destroyed.The
    Resource.DeletePool privilege must be held on the pool as well as the parent of
    the resource pool. Also, the Resource.AssignVMToPool privilege must be held on
    the resource pool's parent pool and any virtual machines that are reassigned.'''

    def __init__(self, core, name=None, ref=None, type=ManagedObjectTypes.ResourcePool):
        super(ResourcePool, self).__init__(core, name=name, ref=ref, type=type)

    
    @property
    def childConfiguration(self):
        '''The resource configuration of all direct children (VirtualMachine and
        ResourcePool) of this resource group.'''
        return self.update('childConfiguration')
    @property
    def config(self):
        '''Configuration of this resource pool.'''
        return self.update('config')
    @property
    def owner(self):
        '''The ComputeResource to which this set of one or more nested resource pools
        belong.'''
        return self.update('owner')
    @property
    def resourcePool(self):
        '''The set of child resource pools.'''
        return self.update('resourcePool')
    @property
    def runtime(self):
        '''Runtime information about a resource pool. The ResourcePoolResourceUsage
        information within ResourcePoolRuntimeInfo can be transiently stale. Use
        RefreshRuntime method to update the information.'''
        return self.update('runtime')
    @property
    def summary(self):
        '''Basic information about a resource pool.'''
        return self.update('summary')
    @property
    def vm(self):
        '''The set of virtual machines associated with this resource pool.'''
        return self.update('vm')

    
    
    def CreateChildVM_Task(self, config, host=None):
        '''Creates a new virtual machine in a vApp container.Creates a new virtual machine
        in a vApp container.Creates a new virtual machine in a vApp container.Creates a
        new virtual machine in a vApp container.Creates a new virtual machine in a vApp
        container.
        
        :param config: The configuration of the virtual machine hardware.
        
        :param host: The target host on which the virtual machine will run. This must specify a host that is a member of the ComputeResource indirectly specified by the pool. For a stand-alone host or a cluster with DRS, host can be omitted, and the system selects a default.
        
        '''
        return self.delegate("CreateChildVM_Task")(config, host)
    
    def CreateResourcePool(self, name, spec):
        '''Creates a new resource pool.Creates a new resource pool.Creates a new resource
        pool.
        
        :param name: 
        
        :param spec: 
        
        '''
        return self.delegate("CreateResourcePool")(name, spec)
    
    def CreateVApp(self, name, resSpec, configSpec, vmFolder=None):
        '''Creates a new vApp container.Creates a new vApp container.
        
        :param name: The name of the vApp container in the inventory
        
        :param resSpec: The resource configuration for the vApp container (same as for a regular resource pool).
        
        :param configSpec: The specification of the vApp specific meta-data.
        
        :param vmFolder: The parent folder for the vApp. This must be null if this is a child vApp.
        
        '''
        return self.delegate("CreateVApp")(name, resSpec, configSpec, vmFolder)
    
    def DestroyChildren(self):
        '''Removes all child resource pools recursively. All virtual machines and vApps
        associated with the child resource pools get associated with this resource
        pool.Removes all child resource pools recursively. All virtual machines and
        vApps associated with the child resource pools get associated with this
        resource pool.Removes all child resource pools recursively. All virtual
        machines and vApps associated with the child resource pools get associated with
        this resource pool.Removes all child resource pools recursively. All virtual
        machines and vApps associated with the child resource pools get associated with
        this resource pool.
        
        '''
        return self.delegate("DestroyChildren")()
    
    def ImportVApp(self, spec, folder=None, host=None):
        '''Creates a new entity in this resource pool. The import process consists of two
        steps:
        
        :param spec: An ImportSpec describing what to import.
        
        :param folder: The folder to which the entity will be attached.
        
        :param host: The target host on which the entity will run. This must specify a host that is a member of the ComputeResource indirectly specified by the pool. For a stand-alone host or a cluster with DRS, host can be omitted, and the system selects a default.
        
        '''
        return self.delegate("ImportVApp")(spec, folder, host)
    
    def MoveIntoResourcePool(self, list):
        '''Moves a set of resource pools, vApps or virtual machines into this pool. The
        pools, vApps and virtual machines must be part of the cluster or standalone
        host that contains this pool.Moves a set of resource pools, vApps or virtual
        machines into this pool. The pools, vApps and virtual machines must be part of
        the cluster or standalone host that contains this pool.Moves a set of resource
        pools, vApps or virtual machines into this pool. The pools, vApps and virtual
        machines must be part of the cluster or standalone host that contains this
        pool.Moves a set of resource pools, vApps or virtual machines into this pool.
        The pools, vApps and virtual machines must be part of the cluster or standalone
        host that contains this pool.Moves a set of resource pools, vApps or virtual
        machines into this pool. The pools, vApps and virtual machines must be part of
        the cluster or standalone host that contains this pool.Moves a set of resource
        pools, vApps or virtual machines into this pool. The pools, vApps and virtual
        machines must be part of the cluster or standalone host that contains this
        pool.
        
        :param list: A list of ResourcePool and VirtualMachine objects.
        
        '''
        return self.delegate("MoveIntoResourcePool")(list)
    
    def QueryResourceConfigOption(self):
        '''Get a value range and default values for ResourceConfigSpec.
        
        '''
        return self.delegate("QueryResourceConfigOption")()
    
    def RefreshRuntime(self):
        '''Refreshes the resource usage data that is available in ResourcePoolRuntimeInfo.
        The latest runtime resource usage of this resource pool may not be available
        immediately after operations that alter resource usage, such as powering on a
        virtual machine. Invoke this method when resource usage may have recently
        changed, and the most up-to-date value in the ResourcePoolRuntimeInfo is
        needed.
        
        '''
        return self.delegate("RefreshRuntime")()
    
    def RegisterChildVM_Task(self, path, name=None, host=None):
        '''Adds an existing virtual machine to this resource pool or vApp.Adds an existing
        virtual machine to this resource pool or vApp.Adds an existing virtual machine
        to this resource pool or vApp.
        
        :param path: A datastore path to the virtual machine. If the path ends with ".vmtx", indicating that it refers to a VM template, an InvalidArgument fault is thrown.
        
        :param name: The name to be assigned to the virtual machine. If this parameter is not set, the displayName configuration parameter of the virtual machine is used. An entity name must be a non-empty string of less than 80 characters. The slash (/), backslash (\) and percent (%) will be escaped using the URL syntax. For example, %2F.
        
        :param host: The target host on which the virtual machine will run. This parameter must specify a host that is a member of the ComputeResource to which this resource pool belongs. For a stand-alone host or a cluster with DRS, the parameter can be omitted, and the system selects a default.
        
        '''
        return self.delegate("RegisterChildVM_Task")(path, name, host)
    
    def UpdateChildResourceConfiguration(self, spec):
        '''Changes resource configuration of a set of children of this resource pool. The
        method allows bulk modifications of the set of the direct children (virtual
        machines and resource pools).Changes resource configuration of a set of
        children of this resource pool. The method allows bulk modifications of the set
        of the direct children (virtual machines and resource pools).Changes resource
        configuration of a set of children of this resource pool. The method allows
        bulk modifications of the set of the direct children (virtual machines and
        resource pools).Changes resource configuration of a set of children of this
        resource pool. The method allows bulk modifications of the set of the direct
        children (virtual machines and resource pools).Changes resource configuration
        of a set of children of this resource pool. The method allows bulk
        modifications of the set of the direct children (virtual machines and resource
        pools).
        
        :param spec: 
        
        '''
        return self.delegate("UpdateChildResourceConfiguration")(spec)
    
    def UpdateConfig(self, name=None, config=None):
        '''Updates the configuration of the resource pool.Updates the configuration of the
        resource pool.Updates the configuration of the resource pool.Updates the
        configuration of the resource pool.
        
        :param name: If set, then the new name of the resource pool.
        
        :param config: If set, then the new resource allocation for this resource pool.
        
        '''
        return self.delegate("UpdateConfig")(name, config)



from pyvisdk.base.managed_object_types import ManagedObjectTypes

from pyvisdk.mo.profile_manager import ProfileManager

import logging

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

class HostProfileManager(ProfileManager):
    '''The HostProfileManager provides access to a list of HostProfiles and it defines
    methods to manipulate profiles and AnswerFiles.'''

    def __init__(self, core, name=None, ref=None, type=ManagedObjectTypes.HostProfileManager):
        super(HostProfileManager, self).__init__(core, name=name, ref=ref, type=type)

    

    
    
    def ApplyHostConfig_Task(self, host, configSpec, userInput=None):
        '''Apply the configuration to the host. If you specify any user input, the
        configuration will be saved in the AnswerFile associated with the host. If
        there is no answer file, the Profile Engine will create one.
        
        :param host: Host to be updated. User must have sufficient credentials and privileges to satisfy the contents of the.
        
        :param configSpec: Set of configuration changes to be applied to the host. The changes are returned by the HostProfile.ExecuteHostProfile method in the ProfileExecuteResult.configSpec property.
        
        :param userInput: Additional host-specific data to be applied to the host. This data is the complete list of deferred parameters verified by the HostProfile.ExecuteHostProfile method, contained in the ProfileExecuteResult object returned by the method.vSphere API 5.0
        
        '''
        return self.delegate("ApplyHostConfig_Task")(host, configSpec, userInput)
    
    def CheckAnswerFileStatus_Task(self, host):
        '''Check the validity of the answer files for the specified hosts. The Profile
        Engine uses the profile associated with a host to check the answer file.
        
        :param host: Set of hosts for which the answer file status will be checked.
        
        '''
        return self.delegate("CheckAnswerFileStatus_Task")(host)
    
    def CreateDefaultProfile(self, profileType, profileTypeName=None, profile=None):
        '''Create a default subprofile of a given type (for example, a
        VirtualSwitchProfile). After you create the subprofile, you can add it to a
        configuration specification and update the host profile:Create a default
        subprofile of a given type (for example, a VirtualSwitchProfile). After you
        create the subprofile, you can add it to a configuration specification and
        update the host profile:
        
        :param profileType: Type of profile to create. The profile types are system-defined (ApplyProfile.profileTypeName).
        
        :param profileTypeName: If specified, the method returns a profile object containing data for the named profile. The type name does not have to be system-defined. A user-defined profile can include various dynamically-defined profiles.vSphere API 5.0
        
        :param profile: Base profile used during the operation.vSphere API 5.0
        
        '''
        return self.delegate("CreateDefaultProfile")(profileType, profileTypeName, profile)
    
    def ExportAnswerFile_Task(self, host):
        '''Export a host's answer file into a serialized form. The method returns a string
        that contains only the list of user input options. See AnswerFile.userInput.
        
        :param host: Host with which the answer file is associated.
        
        '''
        return self.delegate("ExportAnswerFile_Task")(host)
    
    def GenerateConfigTaskList(self, configSpec, host):
        '''Generate a list of configuration tasks that will be performed on the host
        during HostProfile application.
        
        :param configSpec: ConfigSpec which was proposed by ExecuteHostProfile method.
        
        :param host: Host on which the HostProfile application needs to be carried out.
        
        '''
        return self.delegate("GenerateConfigTaskList")(configSpec, host)
    
    def QueryAnswerFileStatus(self, host):
        '''Returns the status of the answer files associated with specified hosts. This
        method returns the most recent status determined by CheckAnswerFileStatus_Task.
        See HostProfileManagerAnswerFileStatus for valid values.
        
        :param host: The hosts the answer file is associated with.
        
        '''
        return self.delegate("QueryAnswerFileStatus")(host)
    
    def QueryHostProfileMetadata(self, profileName=None, profile=None):
        '''Retrieve the metadata for a set of profiles.
        
        :param profileName: Names of the profiles for which metadata is requested. If not set, the method returns metadata for all the profiles.
        
        :param profile: Base profile whose context needs to be used during the operationvSphere API 5.0
        
        '''
        return self.delegate("QueryHostProfileMetadata")(profileName, profile)
    
    def QueryProfileStructure(self, profile=None):
        '''Get information about the structure of the profile.
        
        :param profile: Base profile whose context needs to be used during the operationvSphere API 5.0
        
        '''
        return self.delegate("QueryProfileStructure")(profile)
    
    def RetrieveAnswerFile(self, host):
        '''Returns the answer file associated with a particular host.
        
        :param host: Host with which the answer file is associated.
        
        '''
        return self.delegate("RetrieveAnswerFile")(host)
    
    def UpdateAnswerFile_Task(self, host, configSpec):
        '''Update the AnswerFile for the specified host. If there is no answer file
        associated with the host, the Profile Engine uses the answer file configuration
        specification to create a new one.
        
        :param host: Host with which the answer file is associated.
        
        :param configSpec: Host-specific configuration data. If the configuration specification does not contain any host-specific user input (.userInput), the method does not perform any operation on the answer file.
        
        '''
        return self.delegate("UpdateAnswerFile_Task")(host, configSpec)



import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def VirtualSerialPort(vim, *args, **kwargs):
    '''The data object represents a serial port on a virtual machine. A virtual serial
    port uses one of the following backing types to specify how the virtual machine
    performs serial port operations.When you use network backing, you can also
    configure a virtual serial port to use a virtual serial port concentrator. The
    virtual machine initiates a telnet connection with the concentrator, and the
    concentrator acts as a proxy between the virtual machine and a system on the
    network. By using a virtual serial port concentrator, you can maintain the
    connection between the virtual machine and the network resource when a vMotion
    event moves the virtual machine from one host to another. Without a virtual
    serial port concentrator, the connection would be lost. For information about
    using a serial port concentrator, see .You can configure a virtual serial port
    when you create or reconfigure a virtual machine. For example, to create a
    virtual serial port with network backing, use the following sequence of
    operations. In this procedure, the virtual serial port uses a proxy and will
    accept a network connection.If you use physical device backing ( ), you should
    also use the method to determine if a serial device is available before
    configuring device backing.'''

    obj = vim.client.factory.create('{urn:vim25}VirtualSerialPort')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 2:
        raise IndexError('Expected at least 3 arguments got: %d' % len(args))

    required = [ 'yieldOnPoll', 'key' ]
    optional = [ 'backing', 'connectable', 'controllerKey', 'deviceInfo', 'unitNumber',
        'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




from pyvisdk.base.managed_object_types import ManagedObjectTypes

from pyvisdk.base.base_entity import BaseEntity

import logging

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

class LicenseManager(BaseEntity):
    '''This managed object type controls entitlements for a given VMware platform.
    VMware platforms include VirtualCenter, ESX Server, VMware Server, Workstation
    and Player. Entitlements define what software capabilities this host may
    use.Entitlements are identified by a short string 'key'. Keys can represent
    either a particular edition (Full, Starter) or a particular feature/function
    (featureKey) (backup, nas). An edition implies zero one or more functions which
    are express, denied or optional. For example a 'Full' edition includes 'iscsi'
    function but a Starter edition might disallow it.Which edition a given VMware
    platform uses can be defined at any time. Generally this is done right after
    first install and boot as installation software may not set it. For editions
    that are similar in nature, any future changes to edition type will only impact
    future requests for functionality. Current functionality is left unaffected.
    The same is true for optional functions enabled/disabled after some period of
    time. For dissimilar editions, such transitions may require entering
    maintenance mode first else an exception of InvalidState will be thrown.To
    specify the edition type and any optional functions, use updateLicense for ESX
    Server and addLicense follow by LicenseAssingmentManager.updateAssignedLicense
    for VirtualCenter.When an edition is specified for a given host, the cost of
    that edition (how many licenses are needed) is determined. The cost is computed
    using the license's CostUnit value multiplied by the number of units activated.
    For example, when a VMware platform is set to an edition which uses a
    'cpuPackage' on a two socket server, two licenses would be needed to
    successfully install that edition.Here is a diagram of the unit costs supported
    by this API and their relationships.'''

    def __init__(self, core, name=None, ref=None, type=ManagedObjectTypes.LicenseManager):
        super(LicenseManager, self).__init__(core, name=name, ref=ref, type=type)

    
    @property
    def diagnostics(self):
        '''Return current diagnostic information.'''
        return self.update('diagnostics')
    @property
    def evaluation(self):
        '''vSphere API 4.0'''
        return self.update('evaluation')
    @property
    def featureInfo(self):
        '''The list of features that can be licensed.'''
        return self.update('featureInfo')
    @property
    def licenseAssignmentManager(self):
        '''License Assignment Manager'''
        return self.update('licenseAssignmentManager')
    @property
    def licensedEdition(self):
        '''The product's license edition. The edition defines which product license the
        server requires. This, in turn, determines the core set of functionalities
        provided by the product and the additional features that can be licensed. If no
        edition is set the property is set to the empty string (""). To set the edition
        use SetLicenseEdition.'''
        return self.update('licensedEdition')
    @property
    def licenses(self):
        '''Get information about all the licenses avaiable.'''
        return self.update('licenses')
    @property
    def source(self):
        '''Set or return a data object type of LocalLicense or LicenseServer.'''
        return self.update('source')
    @property
    def sourceAvailable(self):
        '''Current state of the license source. License sources that are LocalSource are
        always available.'''
        return self.update('sourceAvailable')

    
    
    def AddLicense(self, licenseKey, labels=None):
        '''Adds a license to the inventory of available licenses.
        
        :param licenseKey: A license. E.g. a serial license.
        
        :param labels: array of key-value labels. Ignored by ESX Server.
        
        '''
        return self.delegate("AddLicense")(licenseKey, labels)
    
    def CheckLicenseFeature(self, featureKey, host=None):
        '''<b>Deprecated.</b> <i>As of vSphere API 4.0, use QueryAssignedLicenses
        instead.</i> Returns whether or not a given feature is enabled.
        
        :param host: Host to act on if LicenseManager is not on a host.
        
        :param featureKey: Name of the feature to enable.
        
        '''
        return self.delegate("CheckLicenseFeature")(host, featureKey)
    
    def ConfigureLicenseSource(self, licenseSource, host=None):
        '''<b>Deprecated.</b> <i>As of vSphere API 4.0, use UpdateLicense instead.</i>
        Allows for reconfiguration of the License Manager license source.
        
        :param host: Host for which the license manager should be reconfigured.
        
        :param licenseSource: ServedSource or LocalSource.
        
        '''
        return self.delegate("ConfigureLicenseSource")(host, licenseSource)
    
    def DecodeLicense(self, licenseKey):
        '''Decodes licensing information on the license specified.
        
        :param licenseKey: A license. E.g. a serial license.
        
        '''
        return self.delegate("DecodeLicense")(licenseKey)
    
    def DisableFeature(self, featureKey, host=None):
        '''<b>Deprecated.</b> <i>As of vSphere API 4.0, use RemoveAssignedLicense
        instead.</i> Release licenses for an optional feature.
        
        :param host: Host to act on if LicenseManager is not on a host.
        
        :param featureKey: key of the feature to disable.
        
        '''
        return self.delegate("DisableFeature")(host, featureKey)
    
    def EnableFeature(self, featureKey, host=None):
        '''<b>Deprecated.</b> <i>As of vSphere API 4.0, use UpdateAssignedLicense
        instead.</i> Enable a feature that has an optional state.
        
        :param host: Host to act on if LicenseManager is not on a host.
        
        :param featureKey: Name of the feature to enable.
        
        '''
        return self.delegate("EnableFeature")(host, featureKey)
    
    def QueryLicenseSourceAvailability(self, host=None):
        '''<b>Deprecated.</b> <i>As of vSphere API 4.0, use QueryAssignedLicenses
        instead.</i> Queries the current license source for total and available
        licenses available for each feature known to this system.
        
        :param host: Use the license source of the specified host.
        
        '''
        return self.delegate("QueryLicenseSourceAvailability")(host)
    
    def QueryLicenseUsage(self, host=None):
        '''<b>Deprecated.</b> <i>As of vSphere API 4.0, use QueryAssignedLicenses
        instead.</i> Returns the license usage. The license usage is a list of
        supported features and the number of licenses that have been reserved.
        
        :param host: Host to query for usage. If missing, query the server the LicenseManager is on.
        
        '''
        return self.delegate("QueryLicenseUsage")(host)
    
    def QuerySupportedFeatures(self, host=None):
        '''<b>Deprecated.</b> <i>As of vSphere API 4.0, use QueryAssignedLicenses
        instead.</i> Queries the current license source for a list of available
        licenses that can be licensed from this system.
        
        :param host: Use the license source of the specified host.
        
        '''
        return self.delegate("QuerySupportedFeatures")(host)
    
    def RemoveLicense(self, licenseKey):
        '''Remove license from the available set.
        
        :param licenseKey: A licenses. E.g. a serial license.
        
        '''
        return self.delegate("RemoveLicense")(licenseKey)
    
    def RemoveLicenseLabel(self, licenseKey, labelKey):
        '''Removed a license's label.
        
        :param licenseKey: A license.
        
        :param labelKey: A label key.
        
        '''
        return self.delegate("RemoveLicenseLabel")(licenseKey, labelKey)
    
    def SetLicenseEdition(self, host=None, featureKey=None):
        '''<b>Deprecated.</b> <i>As of vSphere API 4.0, use QueryAssignedLicenses
        instead.</i> Defines the product's license edition. The edition defines which
        product license the server requires. This, in turn, determines the core set of
        functionality provided by the product and the additional features that can be
        licensed.
        
        :param host: Host to act on if LicenseManager is not on a host.
        
        :param featureKey: Name of edition feature to select. If featureKey is not set or set to empty string, the product becomes unlicensed.
        
        '''
        return self.delegate("SetLicenseEdition")(host, featureKey)
    
    def UpdateLicense(self, licenseKey, labels=None):
        '''Updates the available licenses to the one provided in licenseKey. This is the
        same as removing all the licenses using RemoveLicense and adding licenseKey
        using AddLicense If the optional parameter labels is specify this is the same
        as calling updateLicense without the optioal parameter and calling updateLabel
        for each pair in the labels array.
        
        :param licenseKey: A license. E.g. a serial license.
        
        :param labels: array of key-value labels.
        
        '''
        return self.delegate("UpdateLicense")(licenseKey, labels)
    
    def UpdateLicenseLabel(self, licenseKey, labelKey, labelValue):
        '''Update a license's label. It creates a label entry if the labelKey doesn't
        already exist
        
        :param licenseKey: A license.
        
        :param labelKey: A label key.
        
        :param labelValue: Value for the label.
        
        '''
        return self.delegate("UpdateLicenseLabel")(licenseKey, labelKey, labelValue)



########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

UpgradePolicy = Enum(
    'manual',
    'upgradeAtPowerCycle',

)




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

FtIssuesOnHostHostSelectionType = Enum(
    'drs',
    'user',
    'vc',

)




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

VirtualMachineDeviceRuntimeInfoVirtualEthernetCardRuntimeStateVmDirectPathGen2InactiveReasonVm = Enum(
    'vmNptConflictingIOChainConfigured',
    'vmNptConflictingOperationInProgress',
    'vmNptDisabledOrDisconnectedAdapter',
    'vmNptFaultToleranceOrRecordReplayConfigured',
    'vmNptIncompatibleAdapterFeatures',
    'vmNptIncompatibleAdapterType',
    'vmNptIncompatibleBackingType',
    'vmNptIncompatibleGuest',
    'vmNptIncompatibleGuestDriver',
    'vmNptInsufficientMemoryReservation',
    'vmNptMonitorBlocks',
    'vmNptOutOfIntrVector',
    'vmNptRuntimeError',

)




import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def WillModifyConfigCpuRequirements(vim, *args, **kwargs):
    '''A virtual machine's total CPU feature requirements are determined by overlaying
    the requirements specified in its configuration (if any) on top of the
    requirements specified in the descriptor for its guest OS. It is therefore
    possible for a host change to implicitly change a virtual machine's CPU feature
    requirements. The guest OS descriptor may have different requirements on the
    new host. Or, if the virtual machine currently specifies requirements in its
    configuration, those requirements will be lost if the new host does not support
    this.This fault indicates that the virtual machine's CPU feature requirements
    would change because of a migration, and also that the destination host does
    support storing CPU feature requirements in the virtual machine's
    configuration. (If the destination host does not support such an action,
    CannotModifyConfigCpuRequirements is used instead of this fault.)This is a
    warning to notify the user that the migration process will adjust the virtual
    machine's configuration so that it will be operating under an unchanged set of
    CPU feature requirements on its new host. If the user wishes to expose the
    different guest OS requirements of the new host, the user will need to edit the
    virtual machine's configuration after the migration.'''

    obj = vim.client.factory.create('{urn:vim25}WillModifyConfigCpuRequirements')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 4:
        raise IndexError('Expected at least 5 arguments got: %d' % len(args))

    required = [ 'dynamicProperty', 'dynamicType', 'faultCause', 'faultMessage' ]
    optional = [  ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




from pyvisdk.esxcli.executer import execute_soap
from pyvisdk.esxcli.base import Base

class StorageNmpPspFixedDeviceconfig(Base):
    '''
    Fixed PSP preferred path configuration operations
    '''
    moid = 'ha-cli-handler-storage-nmp-psp-fixed-deviceconfig'
    def set(self, device, default=None, path=None):
        '''
        Allow setting of the perferred path on a given device controlled by the Fixed Path Selection Policy.
        :param default: boolean, Clear the preferred path selection for the given device.
        :param device: string, The device you wish to set the preferred path for. This device must be controlled by the Fixed Path Selection Policy
        :param path: string, The path you wish to set as the preferred path for the given device.
        :returns: boolean
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.storage.nmp.psp.fixed.deviceconfig.Set',
                            default=default,
                            device=device,
                            path=path,
                            )
    def get(self, device):
        '''
        Allow retrieving of Fixed Path Selection Policy settings for a given device.
        :param device: string, The device you wish to get the Preferred path for.  
        :returns: vim.EsxCLI.storage.nmp.psp.fixed.deviceconfig.get.FixedDeviceConfiguration
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.storage.nmp.psp.fixed.deviceconfig.Get',
                            device=device,
                            )   



########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

BlockDeviceInterface = Enum(
    'fc',
    'fcoe',
    'iscsi',
    'otherBlock',

)




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

VirtualMachineToolsRunningStatus = Enum(
    'guestToolsExecutingScripts',
    'guestToolsNotRunning',
    'guestToolsRunning',

)




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

ReplicationVmConfigFaultReasonForFault = Enum(
    'cannotRetrieveVmReplicationConfiguration',
    'incompatibleHwVersion',
    'invalidDestinationIpAddress',
    'invalidDestinationPort',
    'invalidExtraVmOptions',
    'invalidGenerationNumber',
    'invalidPriorConfiguration',
    'invalidVmReplicationId',
    'outOfBoundsRpoValue',
    'reconfigureVmReplicationIdNotAllowed',
    'replicationAlreadyEnabled',
    'replicationConfigurationFailed',
    'replicationNotEnabled',
    'staleGenerationNumber',

)




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

VirtualMachineToolsVersionStatus = Enum(
    'guestToolsBlacklisted',
    'guestToolsCurrent',
    'guestToolsNeedUpgrade',
    'guestToolsNotInstalled',
    'guestToolsSupportedNew',
    'guestToolsSupportedOld',
    'guestToolsTooNew',
    'guestToolsTooOld',
    'guestToolsUnmanaged',

)




import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def VirtualUSBController(vim, *args, **kwargs):
    '''The VirtualUSBController data object describes a virtual USB controller and
    contains a list of the devices connected to the controller. A virtual machine
    must have a virtual USB controller before you can add a USB device to the
    virtual machine configuration. To add a controller, include a
    VirtualUSBController object in the VirtualDeviceConfigSpec for your virtual
    machine configuration. You can add only one controller to a virtual machine. A
    virtual USB controller supports up to 20 USB device connections on the virtual
    machine.The ESX Server host must have the USB controller hardware and modules
    that support USB 2.0 and USB1.1. You can use a maximum of 15 USB controllers on
    a host. If your system includes an additional number of controllers with
    connected devices, the additional devices will not be available to virtual
    machines on the host.You must remove all USB devices from a virtual machine
    before you can remove the USB controller.'''

    obj = vim.client.factory.create('{urn:vim25}VirtualUSBController')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 2:
        raise IndexError('Expected at least 3 arguments got: %d' % len(args))

    required = [ 'busNumber', 'key' ]
    optional = [ 'autoConnectDevices', 'ehciEnabled', 'device', 'backing', 'connectable',
        'controllerKey', 'deviceInfo', 'unitNumber', 'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




from pyvisdk.esxcli.executer import execute_soap
from pyvisdk.esxcli.base import Base

class StorageVmfs(Base):
    '''
    VMFS operations.
    '''
    moid = 'ha-cli-handler-storage-vmfs'
    def upgrade(self, volumelabel=None, volumeuuid=None):
        '''
        Upgrade a VMFS3 volume to VMFS5.
        :param volumelabel: string, The label of the volume to upgrade.
        :param volumeuuid: string, The uuid of the volume to upgrade.
        :returns: boolean
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.storage.vmfs.Upgrade',
                            volumelabel=volumelabel,
                            volumeuuid=volumeuuid,
                            )   



import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def HostActiveDirectory(vim, *args, **kwargs):
    '''The HostActiveDirectory data object contains Active Directory configuration
    information for an ESX host.The vSphere API supports Microsoft Active Directory
    management of authentication for ESX hosts. To integrate an ESX host into an
    Active Directory environment, you use an Active Directory account that has the
    authority to add a computer to a domain. The ESX Server locates the Active
    Directory domain controller. When you use the host profile to configure
    authentication for an ESX host, you specify the configuration operation (add or
    remove). To add the host to a domain, specify the domain, and the authorized
    Active Directory account user name and password. You do not need to specify
    these parameters to remove the host from a domain because the host has the
    information it needs to perform the operation. When you call
    ApplyHostConfig_Task to apply the configuration, the ESX Server will call the
    appropriate method (JoinDomain_Task or LeaveCurrentDomain_Task) on your
    behalf.Before you call the method to apply the host profile, check to see that
    the HostAuthenticationManager.supportedStore array contains a
    HostActiveDirectoryAuthentication object. The presence of the Active Directory
    authentication object indicates that a host is capable of joining a domain.
    However, if you try to add a host to a domain when the
    HostAuthenticationStoreInfo.enabled property is , the join method will throw a
    fault.As an alternative to using the host profile to configure Active Directory
    authentication for an ESX host, your vSphere client application can call the
    HostActiveDirectoryAuthentication join and leave methods directly to add the
    host to or remove the host from a domain.To take advantage of ESX host
    membership in an Active Directory domain, grant permissions on the ESX host to
    users and groups in Active Directory who should have direct access to
    management of the ESX host. Use the UserDirectory.RetrieveUserGroups method to
    obtain information about Active Directory users and groups. After retrieving
    the Active Directory data, you can use the
    AuthorizationManager.SetEntityPermissions method to set the principal property
    to the appropriate user or group.By default, the ESX host assigns the
    Administrator role to the "ESX Admins" group. If the group does not exist when
    the host joins the domain, the host will not assign the role. In this case, you
    must create the "ESX Admins" group in the Active Directory. The host will
    periodically check the domain controller for the group and will assign the role
    when the group exists.'''

    obj = vim.client.factory.create('{urn:vim25}HostActiveDirectory')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 1:
        raise IndexError('Expected at least 2 arguments got: %d' % len(args))

    required = [ 'changeOperation' ]
    optional = [ 'spec', 'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




from pyvisdk.base.managed_object_types import ManagedObjectTypes

from pyvisdk.mo.extensible_managed_object import ExtensibleManagedObject

import logging

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

class ManagedEntity(ExtensibleManagedObject):
    '''ManagedEntity is an abstract base type for all managed objects present in the
    inventory tree. The base type provides common functionality for traversing the
    tree structure, as well as health monitoring and other basic functions.Most
    Virtual Infrastructure managed object types extend this type.'''

    def __init__(self, core, name=None, ref=None, type=ManagedObjectTypes.ManagedEntity):
        super(ManagedEntity, self).__init__(core, name=name, ref=ref, type=type)

    
    @property
    def alarmActionsEnabled(self):
        '''Whether alarm actions are enabled for this entity. True if enabled; false
        otherwise.'''
        return self.update('alarmActionsEnabled')
    @property
    def configIssue(self):
        '''Current configuration issues that have been detected for this entity.
        Typically, these issues have already been logged as events. The entity stores
        these events as long as they are still current. The configStatus property
        provides an overall status based on these events.'''
        return self.update('configIssue')
    @property
    def configStatus(self):
        '''The configStatus indicates whether or not the system has detected a
        configuration issue involving this entity. For example, it might have detected
        a duplicate IP address or MAC address, or a host in a cluster might be out of
        compliance. The meanings of the configStatus values are: * red: A problem has
        been detected involving the entity. * yellow: A problem is about to occur or a
        transient condition has occurred (For example, reconfigure fail-over policy). *
        green: No configuration issues have been detected. * gray: The configuration
        status of the entity is not being monitored. A green status indicates only that
        a problem has not been detected; it is not a guarantee that the entity is
        problem-free.'''
        return self.update('configStatus')
    @property
    def customValue(self):
        '''Custom field values.'''
        return self.update('customValue')
    @property
    def declaredAlarmState(self):
        '''A set of alarm states for alarms that apply to this managed entity. The set
        includes alarms defined on this entity and alarms inherited from the parent
        entity, or from any ancestors in the inventory hierarchy.'''
        return self.update('declaredAlarmState')
    @property
    def disabledMethod(self):
        '''List of operations that are disabled, given the current runtime state of the
        entity. For example, a power-on operation always fails if a virtual machine is
        already powered on. This list can be used by clients to enable or disable
        operations in a graphical user interface.'''
        return self.update('disabledMethod')
    @property
    def effectiveRole(self):
        '''Access rights the current session has to this entity.'''
        return self.update('effectiveRole')
    @property
    def name(self):
        '''Name of this entity, unique relative to its parent.'''
        return self.update('name')
    @property
    def overallStatus(self):
        '''General health of this managed entity. The overall status of the managed entity
        is computed as the worst status among its alarms and the configuration issues
        detected on the entity. The status is reported as one of the following values:
        * red: The entity has alarms or configuration issues with a red status. *
        yellow: The entity does not have alarms or configuration issues with a red
        status, and has at least one with a yellow status. * green: The entity does not
        have alarms or configuration issues with a red or yellow status, and has at
        least one with a green status. * gray: All of the entity's alarms have a gray
        status and the configuration status of the entity is not being monitored.'''
        return self.update('overallStatus')
    @property
    def parent(self):
        '''Parent of this entity.'''
        return self.update('parent')
    @property
    def permission(self):
        '''List of permissions defined for this entity.'''
        return self.update('permission')
    @property
    def recentTask(self):
        '''The set of recent tasks operating on this managed entity. This is a subset of
        recentTask belong to this entity. A task in this list could be in one of the
        four states: pending, running, success or error.'''
        return self.update('recentTask')
    @property
    def tag(self):
        '''The set of tags associated with this managed entity. Experimental. Subject to
        change.'''
        return self.update('tag')
    @property
    def triggeredAlarmState(self):
        '''A set of alarm states for alarms triggered by this entity or by its
        descendants.'''
        return self.update('triggeredAlarmState')

    
    
    def Destroy_Task(self):
        '''Destroys this object, deleting its contents and removing it from its parent
        folder (if any).Destroys this object, deleting its contents and removing it
        from its parent folder (if any).Destroys this object, deleting its contents and
        removing it from its parent folder (if any).
        
        '''
        return self.delegate("Destroy_Task")()
    
    def Reload(self):
        '''Reload the entity state. Clients only need to call this method if they changed
        some external state that affects the service without using the Web service
        interface to perform the change. For example, hand-editing a virtual machine
        configuration file affects the configuration of the associated virtual machine
        but the service managing the virtual machine might not monitor the file for
        changes. In this case, after such an edit, a client would call "reload" on the
        associated virtual machine to ensure the service and its clients have current
        data for the virtual machine.
        
        '''
        return self.delegate("Reload")()
    
    def Rename_Task(self, newName):
        '''Renames this managed entity.Renames this managed entity.Renames this managed
        entity.
        
        :param newName: See name
        
        '''
        return self.delegate("Rename_Task")(newName)



from pyvisdk.esxcli.executer import execute_soap
from pyvisdk.esxcli.base import Base

class SystemVersion(Base):
    '''
    Commands to get version information.
    '''
    moid = 'ha-cli-handler-system-version'
    def get(self):
        '''
        Display the product name, version and build information.
        :returns: vim.EsxCLI.system.version.get.VersionGet
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.system.version.Get',
                            )   



########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

ReplicationVmState = Enum(
    'active',
    'error',
    'idle',
    'none',
    'paused',
    'syncing',

)




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

HostVmciAccessManagerMode = Enum(
    'grant',
    'replace',
    'revoke',

)




from pyvisdk.base.managed_object_types import ManagedObjectTypes

from pyvisdk.base.base_entity import BaseEntity

import logging

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

class PropertyFilter(BaseEntity):
    '''The PropertyFilter managed object type defines a filter that controls the
    properties for which a PropertyCollector detects incremental changes. Filters
    are subordinate objects; they are part of the PropertyCollector and do not have
    independent lifetimes. A Filter is automatically destroyed when the session on
    which it was created is closed or the PropertyCollector on which it was created
    is destroyed.'''

    def __init__(self, core, name=None, ref=None, type=ManagedObjectTypes.PropertyFilter):
        super(PropertyFilter, self).__init__(core, name=name, ref=ref, type=type)

    
    @property
    def partialUpdates(self):
        '''Flag to indicate if a change to a nested property reports only the nested
        change or the entire specified property value. If the value is true, a change
        reports only the nested property. If the value is false, a change reports the
        enclosing property named in the filter.'''
        return self.update('partialUpdates')
    @property
    def spec(self):
        '''Specifications for this filter.'''
        return self.update('spec')

    
    
    def DestroyPropertyFilter(self):
        '''Destroys this filter.Destroys this filter.
        
        '''
        return self.delegate("DestroyPropertyFilter")()



import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def VirtualDiskBlocksNotFullyProvisioned(vim, *args, **kwargs):
    '''The disk blocks of the specified virtual disk have not been fully provisioned
    on the file system.Typically, this fault is returned as part of a parent fault
    like VmConfigIncompatibleForFaultTolerance, indicating that the disk blocks of
    the virtual disk must be fully provisioned on the file system before fault
    tolerance can be enabled on the associated virtual machine.'''

    obj = vim.client.factory.create('{urn:vim25}VirtualDiskBlocksNotFullyProvisioned')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 7:
        raise IndexError('Expected at least 8 arguments got: %d' % len(args))

    required = [ 'backing', 'device', 'reason', 'dynamicProperty', 'dynamicType', 'faultCause',
        'faultMessage' ]
    optional = [  ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




from pyvisdk.base.managed_object_types import ManagedObjectTypes

from pyvisdk.mo.host_authentication_store import HostAuthenticationStore

import logging

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

class HostLocalAuthentication(HostAuthenticationStore):
    '''The HostLocalAuthentication managed object represents local authentication for
    user accounts on an ESX host.'''

    def __init__(self, core, name=None, ref=None, type=ManagedObjectTypes.HostLocalAuthentication):
        super(HostLocalAuthentication, self).__init__(core, name=name, ref=ref, type=type)

    

    



import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def ClusterFailoverResourcesAdmissionControlPolicy(vim, *args, **kwargs):
    '''The ClusterFailoverResourcesAdmissionControlPolicy reserves a specified
    percentage of aggregate cluster resources for failover. With the resources
    failover policy in place, vSphere HA uses the following calculations to control
    virtual machine migration in the cluster.HA uses the actual reservations of the
    virtual machines. If a virtual machine does not have reservations, meaning that
    the reservation is 0, a default of 0MB memory and 256MHz CPU is applied. This
    is controlled by the same HA advanced options used for the failover level
    policy (ClusterFailoverLevelAdmissionControlPolicy).'''

    obj = vim.client.factory.create('{urn:vim25}ClusterFailoverResourcesAdmissionControlPolicy')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 2:
        raise IndexError('Expected at least 3 arguments got: %d' % len(args))

    required = [ 'cpuFailoverResourcesPercent', 'memoryFailoverResourcesPercent' ]
    optional = [ 'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




import logging
from pyvisdk.exceptions import InvalidArgumentError

# This module is NOT auto-generated
# Inspired by decompiled Java classes from vCenter's internalvim25stubs.jar
# Unless states otherside, the methods and attributes were not used by esxcli,
# and thus not tested

log = logging.getLogger(__name__)

def VimEsxCLInetworkipgetIPInfo(vim, *args, **kwargs):
    obj = vim.client.factory.create('{urn:vim25}VimEsxCLInetworkipgetIPInfo')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 0:
        raise IndexError('Expected at least 1 arguments got: %d' % len(args))

    required = [  ]
    optional = [ 'IPv6Enabled' ]

    for name, arg in zip(required + optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj



import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def ConfigTarget(vim, *args, **kwargs):
    '''The ConfigTarget class contains information about "physical" devices that can
    be used to back virtual devices.'''

    obj = vim.client.factory.create('{urn:vim25}ConfigTarget')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 5:
        raise IndexError('Expected at least 6 arguments got: %d' % len(args))

    required = [ 'maxMemMBOptimalPerf', 'numCpuCores', 'numCpus', 'numNumaNodes', 'smcPresent' ]
    optional = [ 'autoVmotion', 'cdRom', 'datastore', 'distributedVirtualPortgroup',
        'distributedVirtualSwitch', 'floppy', 'ideDisk', 'legacyNetworkInfo',
        'network', 'parallel', 'pciPassthrough', 'resourcePool', 'scsiDisk',
        'scsiPassthrough', 'serial', 'sound', 'usb', 'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def ServiceConsoleReservationInfo(vim, *args, **kwargs):
    '''The ServiceConsoleReservationInfo data object type describes the amount of
    memory that is being reserved by the service console. Memory reserved for use
    by the service console is a hard reservation that cannot be changed except
    across hardware restarts.This memory that is reserved for the service console
    is used primarily to provide system management services. In addition, a small
    overhead is needed by each virtual machine on the service console.The only
    property of the data object that can be changed directly is the
    serviceConsoleReservedCfg property. This property indicates how much memory
    should be reserved for the service console on the next boot. In most cases,
    this amount is the same as the current reservation.'''

    obj = vim.client.factory.create('{urn:vim25}ServiceConsoleReservationInfo')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 3:
        raise IndexError('Expected at least 4 arguments got: %d' % len(args))

    required = [ 'serviceConsoleReserved', 'serviceConsoleReservedCfg', 'unreserved' ]
    optional = [ 'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def OptionProfile(vim, *args, **kwargs):
    '''The OptionProfile data object encapsulates one advanced configuration. Use the
    policy list for access to configuration data for the option profile. Use the
    property list for access to subprofile configuration data, if any.'''

    obj = vim.client.factory.create('{urn:vim25}OptionProfile')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 2:
        raise IndexError('Expected at least 3 arguments got: %d' % len(args))

    required = [ 'key', 'enabled' ]
    optional = [ 'policy', 'profileTypeName', 'profileVersion', 'property', 'dynamicProperty',
        'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




from pyvisdk.base.managed_object_types import ManagedObjectTypes

from pyvisdk.base.base_entity import BaseEntity

import logging

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

class ExtensionManager(BaseEntity):
    '''This managed object type provides directory and basic management services for
    all registered extensions.Clients use the ExtensionManager, available in
    ServiceInstance, to access extension objects.While several authentication
    methods are available for extension servers to use (see SessionManager), only
    one authentication method is valid for an extension at any given time.'''

    def __init__(self, core, name=None, ref=None, type=ManagedObjectTypes.ExtensionManager):
        super(ExtensionManager, self).__init__(core, name=name, ref=ref, type=type)

    
    @property
    def extensionList(self):
        '''The list of currently registered extensions.'''
        return self.update('extensionList')

    
    
    def FindExtension(self, extensionKey):
        '''Returns extension with the given key, if any.
        
        :param extensionKey: Key to search for.
        
        '''
        return self.delegate("FindExtension")(extensionKey)
    
    def GetPublicKey(self):
        '''<b>Deprecated.</b> <i>As of VI 4.0, use trusted certificates and
        LoginExtensionBySubjectName or SetExtensionCertificate and
        LoginExtensionByCertificate.</i> Returns VirtualCenter Server public key.
        
        '''
        return self.delegate("GetPublicKey")()
    
    def QueryManagedBy(self, extensionKey):
        '''Find entities managed by an extension. These can be either virtual machines or
        vApps.
        
        :param extensionKey: Key of the extension to find managed entities for.
        
        '''
        return self.delegate("QueryManagedBy")(extensionKey)
    
    def RegisterExtension(self, extension):
        '''Registers extension.
        
        :param extension: Extension description to register.
        
        '''
        return self.delegate("RegisterExtension")(extension)
    
    def SetExtensionCertificate(self, extensionKey, certificatePem=None):
        '''Update the stored authentication certificate for a specified extension. Updates
        the registration of the specified extension with the thumbprint of the X.509
        client certificate provided over SSL handshake, or by the
        "certificatePem"argument. The thumbprint will be used to authenticate the
        extension during invocations of LoginExtensionByCertificate.Update the stored
        authentication certificate for a specified extension. Updates the registration
        of the specified extension with the thumbprint of the X.509 client certificate
        provided over SSL handshake, or by the "certificatePem"argument. The thumbprint
        will be used to authenticate the extension during invocations of
        LoginExtensionByCertificate.Update the stored authentication certificate for a
        specified extension. Updates the registration of the specified extension with
        the thumbprint of the X.509 client certificate provided over SSL handshake, or
        by the "certificatePem"argument. The thumbprint will be used to authenticate
        the extension during invocations of LoginExtensionByCertificate.
        
        :param extensionKey: Key of extension to update.
        
        :param certificatePem: PEM encoded certificate. If not specified, the certificate passed over SSL handshake is used.
        
        '''
        return self.delegate("SetExtensionCertificate")(extensionKey, certificatePem)
    
    def SetPublicKey(self, extensionKey, publicKey):
        '''<b>Deprecated.</b> <i>As of VI 4.0, use trusted certificates and
        LoginExtensionBySubjectName or SetExtensionCertificate and
        LoginExtensionByCertificate.</i> Sets extension's public key.
        
        :param extensionKey: Key of extension to update.
        
        :param publicKey: Public key of extension, encoded in PEM (privacy-enhanced mail) format.
        
        '''
        return self.delegate("SetPublicKey")(extensionKey, publicKey)
    
    def UnregisterExtension(self, extensionKey):
        '''Unregisters the specified extension if it exists.
        
        :param extensionKey: Unique name of extension to unregister.
        
        '''
        return self.delegate("UnregisterExtension")(extensionKey)
    
    def UpdateExtension(self, extension):
        '''If the key specified in the extension exists, the existing record is updated.If
        the key specified in the extension exists, the existing record is updated.
        
        :param extension: Updated extension description.
        
        '''
        return self.delegate("UpdateExtension")(extension)



import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def HostNetworkPolicy(vim, *args, **kwargs):
    '''This data object type describes network policies that can be configured for
    both virtual switches and port groups. The policy settings on the port group
    can inherit policy settings from their containing virtual switch. These policy
    settings are inherited if the settings on the port group are not set. Since
    every policy setting on a port group is optional, every individual policy
    setting can be inherited.By contrast, if a host is capable of implementing a
    policy setting, every virtual switch has some value assigned to the policy
    setting. In this case, although all of the policy settings are optional, they
    always have some value either by inheritance or by direct setting.Policy
    settings are organized into policy groups such as SecurityPolicy. Policy groups
    are optional since it is possible that a host may not implement such policies.
    If a host does not support a policy group, the policy group is not set on both
    the virtual switches and the port groups.See HostNetCapabilities'''

    obj = vim.client.factory.create('{urn:vim25}HostNetworkPolicy')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 0:
        raise IndexError('Expected at least 1 arguments got: %d' % len(args))

    required = [  ]
    optional = [ 'nicTeaming', 'offloadPolicy', 'security', 'shapingPolicy', 'dynamicProperty',
        'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

VirtualMachineScsiPassthroughType = Enum(
    'cdrom',
    'com',
    'disk',
    'media',
    'optical',
    'printer',
    'processor',
    'raid',
    'scanner',
    'tape',
    'unknown',
    'worm',

)
